@@ -26,6 +26,30 @@
 
 #include <stdbool.h>
 
+extern const struct intel_execution_ring {
+	const char *name;
+	const char *full_name;
+	unsigned exec_id;
+	unsigned flags;
+} intel_execution_rings[];
+
+#define eb_ring(e) ((e)->exec_id | (e)->flags)
+
+#define for_each_ring(it__, fd__) \
+	for (const struct intel_execution_ring *it__ = intel_execution_rings;\
+	     it__->name; \
+	     it__++) \
+		for_if (gem_has_ring(fd__, eb_ring(it__)))
+
+#define for_each_physical_ring(it__, fd__) \
+	for (const struct intel_execution_ring *it__ = intel_execution_rings;\
+	     it__->name; \
+	     it__++) \
+		for_if (gem_ring_has_physical_engine(fd__, eb_ring(it__)))
+
+bool gem_ring_is_physical_engine(int fd, unsigned int ring);
+bool gem_ring_has_physical_engine(int fd, unsigned int ring);
+
 enum measure_ring_flags {
 	MEASURE_RING_NEW_CTX = 1
 };
