@@ -26,7 +26,6 @@
 
 #include "drmtest.h"
 #include "i915_3d.h"
-#include "i915_pciids.h"
 #include "igt_aux.h"
 #include "igt_core.h"
 #include "igt_core.h"
@@ -38,8 +37,10 @@
 #include "igt_gt.h"
 #include "igt_kms.h"
 #include "igt_params.h"
+#include "igt_pipe_crc.h"
 #include "igt_pm.h"
 #include "igt_stats.h"
+#include "igt_dsc.h"
 #ifdef HAVE_CHAMELIUM
 #include "igt_alsa.h"
 #include "igt_audio.h"
