@@ -136,8 +136,8 @@ struct _GKeyFile *igt_load_igtrc(void);
 /**
  * IGT_EXIT_ABORT
  *
- * Exit status indicating a severe test/enviroment failure, any continued
- * testing past this point can yeild unexpected reasults and is not recommended
+ * Exit status indicating a severe test/environment failure, any continued
+ * testing past this point can yield unexpected results and is not recommended
  */
 #define IGT_EXIT_ABORT 112
 
@@ -1448,6 +1448,8 @@ int igt_system_quiet(const char *command);
 		free(buf); \
 	} while (0)
 
+void igt_srandom(void);
+
 /**
  * igt_trace:
  * @format: printf-style format string with optional args
