@@ -785,6 +785,11 @@ bool igt_wait_for_pm_status(enum igt_runtime_pm_status status)
 	return ret;
 }
 
+static const char *yesno(bool x)
+{
+	return x ? "yes" : "no";
+}
+
 /**
  * dmc_loaded:
  * @debugfs: fd to the debugfs dir.
@@ -798,7 +803,8 @@ bool igt_wait_for_pm_status(enum igt_runtime_pm_status status)
  */
 bool igt_pm_dmc_loaded(int debugfs)
 {
-	char buf[15];
+	char buf[512];
+	bool loaded;
 	int len;
 
 	len = igt_sysfs_read(debugfs, "i915_dmc_info", buf, sizeof(buf) - 1);
@@ -807,8 +813,10 @@ bool igt_pm_dmc_loaded(int debugfs)
 
 	buf[len] = '\0';
 
-	igt_info("DMC: %s\n", buf);
-	return strstr(buf, "fw loaded: yes");
+	loaded = strstr(buf, "fw loaded: yes");
+	igt_info("DMC: fw loaded: %s\n", yesno(loaded));
+
+	return loaded;
 }
 
 /**
@@ -1132,9 +1140,6 @@ igt_pm_setup_pci_card_power_attrs(struct pci_device *pci_dev, bool save_attrs, i
 	ret = pci_device_get_bridge_buses(pci_dev, &primary, &secondary, &subordinate);
 	igt_assert(!ret);
 
-	ret = pci_system_init();
-	igt_assert(!ret);
-
 	match.domain = pci_dev->domain;
 	match.bus = PCI_MATCH_ANY;
 	match.dev = PCI_MATCH_ANY;
@@ -1177,7 +1182,6 @@ void igt_pm_enable_pci_card_runtime_pm(struct pci_device *root,
 		delay = igt_pm_get_autosuspend_delay(i915);
 
 	igt_pm_setup_pci_card_power_attrs(root, false, delay);
-	pci_system_cleanup();
 }
 
 /**
@@ -1196,47 +1200,47 @@ void igt_pm_setup_pci_card_runtime_pm(struct pci_device *pci_dev)
 
 /**
  * igt_pm_get_d3cold_allowed:
- * @igt_device_card: device card
- * @val: value to be read into
+ * @pci_slot_name: slot name of the pci device
+ * @value: value to be read into
  *
  * Reads the value of d3cold_allowed attribute
  * of the pci device
  */
-void igt_pm_get_d3cold_allowed(struct igt_device_card *card, char *val)
+void igt_pm_get_d3cold_allowed(const char *pci_slot_name, uint32_t *value)
 {
 	char name[PATH_MAX];
 	int fd;
 
 	snprintf(name, PATH_MAX, "/sys/bus/pci/devices/%s",
-		 card->pci_slot_name);
+		 pci_slot_name);
 
 	fd = open(name, O_RDONLY);
 	igt_assert_f(fd >= 0, "Can't open %s\n", name);
 
-	igt_sysfs_read(fd, "d3cold_allowed", val, sizeof(val));
+	__igt_sysfs_get_u32(fd, "d3cold_allowed", value);
 
 	close(fd);
 }
 
 /**
- * igt_pm_get_d3cold_allowed:
- * @igt_device_card: device card
- * @val: value to be written
+ * igt_pm_set_d3cold_allowed:
+ * @pci_slot_name: slot name of pci device
+ * @value: value to be written
  *
  * writes the value to d3cold_allowed attribute of pci device
  */
-void igt_pm_set_d3cold_allowed(struct igt_device_card *card, const char *val)
+void igt_pm_set_d3cold_allowed(const char *pci_slot_name, uint32_t value)
 {
 	char name[PATH_MAX];
 	int fd;
 
 	snprintf(name, PATH_MAX, "/sys/bus/pci/devices/%s",
-		 card->pci_slot_name);
+		 pci_slot_name);
 
 	fd = open(name, O_RDONLY);
 	igt_assert_f(fd >= 0, "Can't open %s\n", name);
 
-	igt_sysfs_write(fd, "d3cold_allowed", val, sizeof(val));
+	__igt_sysfs_set_u32(fd, "d3cold_allowed", value);
 	close(fd);
 }
 
@@ -1281,7 +1285,6 @@ void igt_pm_restore_pci_card_runtime_pm(void)
 	}
 
 	memset(__pci_dev_pwrattr, 0, sizeof(__pci_dev_pwrattr));
-	pci_system_cleanup();
 }
 
 static void igt_pm_print_pci_dev_runtime_status(struct pci_device *pci_dev)
@@ -1321,21 +1324,40 @@ void igt_pm_print_pci_card_runtime_status(void)
 	}
 }
 
-bool i915_is_slpc_enabled(int fd)
+/**
+ * i915_is_slpc_enabled_gt:
+ * @drm_fd: DRM file descriptor
+ * @gt: GT id
+ * Check if SLPC is enabled on a GT
+ */
+bool i915_is_slpc_enabled_gt(int drm_fd, int gt)
 {
-	int debugfs_fd = igt_debugfs_dir(fd);
+	int dir, debugfs_fd;
 	char buf[4096] = {};
-	int len;
 
-	igt_require(debugfs_fd != -1);
+	dir = igt_debugfs_gt_dir(drm_fd, gt);
+	igt_require(dir);
+
+	debugfs_fd = openat(dir, "uc/guc_slpc_info", O_RDONLY);
+	/* if guc_slpc_info not present then return false */
+	if (debugfs_fd < 0)
+		return false;
+
+	read(debugfs_fd, buf, sizeof(buf)-1);
 
-	len = igt_debugfs_simple_read(debugfs_fd, "gt/uc/guc_slpc_info", buf, sizeof(buf));
 	close(debugfs_fd);
 
-	if (len < 0)
-		return false;
-	else
-		return strstr(buf, "SLPC state: running");
+	return strstr(buf, "SLPC state: running");
+}
+
+/**
+ * i915_is_slpc_enabled:
+ * @drm_fd: DRM file descriptor
+ * Check if SLPC is enabled for the device
+ */
+bool i915_is_slpc_enabled(int drm_fd)
+{
+	return i915_is_slpc_enabled_gt(drm_fd, 0);
 }
 
 int igt_pm_get_runtime_suspended_time(struct pci_device *pci_dev)
@@ -1355,3 +1377,38 @@ int igt_pm_get_runtime_suspended_time(struct pci_device *pci_dev)
 
 	return -1;
 }
+
+/**
+ * igt_pm_get_runtime_usage:
+ * @pci_dev: pci device
+ *
+ * Reports the runtime PM usage count of a device.
+ */
+int igt_pm_get_runtime_usage(struct pci_device *pci_dev)
+{
+	char usage_str[64];
+	int usage, fd;
+
+	fd = igt_pm_get_power_attr_fd_rdonly(pci_dev, "runtime_usage");
+	if (igt_pm_read_power_attr(fd, usage_str, 64, true))
+		igt_assert(sscanf(usage_str, "%d", &usage) > 0);
+
+	return usage;
+}
+
+/**
+ * igt_pm_ignore_slpc_efficient_freq
+ * @i915: open i915 drm file descriptor
+ * @gtfd: open gt sysfs fd
+ * @val: value to set
+ *
+ * Ignores/un-ignores SLPC efficient frequency
+ */
+void igt_pm_ignore_slpc_efficient_freq(int i915, int gtfd, bool val)
+{
+	if (!(gem_using_guc_submission(i915) && i915_is_slpc_enabled(i915)))
+		return;
+
+	igt_require(igt_sysfs_has_attr(gtfd, "slpc_ignore_eff_freq"));
+	igt_sysfs_set_u32(gtfd, "slpc_ignore_eff_freq", val);
+}
