@@ -24,6 +24,12 @@
  *    Lyude Paul <lyude@redhat.com>
  */
 
+/**
+ * TEST: kms chamelium hpd
+ * Category: Display
+ * Description: Testing HPD with a Chamelium board
+ */
+
 #include "kms_chamelium_helper.h"
 
 #define HPD_STORM_PULSE_INTERVAL_DP 100 /* ms */
@@ -104,6 +110,84 @@ static void try_suspend_resume_hpd(chamelium_data_t *data,
 	}
 }
 
+/**
+ * SUBTEST: dp-hpd-fast
+ * Description: Check that we get uevents and updated connector status on
+ * 		hotplug and unplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: DP
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: hdmi-hpd-fast
+ * Description: Check that we get uevents and updated connector status on
+ * 		hotplug and unplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: HDMI
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: vga-hpd-fast
+ * Description: Check that we get uevents and updated connector status on
+ * 		hotplug and unplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: VGA
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: dp-hpd
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: DP
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: hdmi-hpd
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: HDMI
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: vga-hpd
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: VGA
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: dp-hpd-%s
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: DP
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: hdmi-hpd-%s
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: HDMI
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: vga-hpd-%s
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: VGA
+ * Driver requirement: i915, xe
+ *
+ * arg[1]:
+ *
+ * @enable-disable-mode:     Toggle the mode
+ * @with-enabled-mode:       Enabling the mode
+ */
 static const char test_basic_hotplug_desc[] =
 	"Check that we get uevents and updated connector status on "
 	"hotplug and unplug";
@@ -170,6 +254,31 @@ static void test_hotplug(chamelium_data_t *data, struct chamelium_port *port,
 	igt_remove_fb(data->drm_fd, &fb);
 }
 
+/**
+ * SUBTEST: dp-hpd-for-each-pipe
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug for each pipe with valid output
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: DP
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: hdmi-hpd-for-each-pipe
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug for each pipe with valid output
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: HDMI
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: vga-hpd-for-each-pipe
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug for each pipe with valid output
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: VGA
+ * Driver requirement: i915, xe
+ */
 static const char test_hotplug_for_each_pipe_desc[] =
 	"Check that we get uevents and updated connector status on "
 	"hotplug and unplug for each pipe with valid output";
@@ -190,6 +299,7 @@ static void test_hotplug_for_each_pipe(chamelium_data_t *data,
 						   DRM_MODE_DISCONNECTED);
 
 	for_each_pipe(&data->display, pipe) {
+		igt_modeset_disable_all_outputs(&data->display);
 		igt_flush_uevents(mon);
 		/* Check if we get a sysfs hotplug event */
 		chamelium_plug(data->chamelium, port);
@@ -214,6 +324,55 @@ static void test_hotplug_for_each_pipe(chamelium_data_t *data,
 	igt_hpd_storm_reset(data->drm_fd);
 }
 
+/**
+ * SUBTEST: dp-hpd-after-hibernate
+ * Description: Toggle HPD during Hibernation, check that uevents are sent and
+ *              connector status is updated
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug, hibernation
+ * Mega feature: DP
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: hdmi-hpd-after-hibernate
+ * Description: Toggle HPD during Hibernation, check that uevents are sent and
+ *              connector status is updated
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug, hibernation
+ * Mega feature: HDMI
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: vga-hpd-after-hibernate
+ * Description: Toggle HPD during Hibernation, check that uevents are sent and
+ *              connector status is updated
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug, hibernation
+ * Mega feature: VGA
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: dp-hpd-after-suspend
+ * Description: Toggle HPD during Suspend, check that uevents are sent and
+ *              connector status is updated
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug, suspend
+ * Mega feature: DP
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: hdmi-hpd-after-suspend
+ * Description: Toggle HPD during Suspend, check that uevents are sent and
+ *              connector status is updated
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug, suspend
+ * Mega feature: HDMI
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: vga-hpd-after-suspend
+ * Description: Toggle HPD during Suspend, check that uevents are sent and
+ *              connector status is updated
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug, suspend
+ * Mega feature: VGA
+ * Driver requirement: i915, xe
+ */
 static const char test_suspend_resume_hpd_desc[] =
 	"Toggle HPD during suspend, check that uevents are sent and connector "
 	"status is updated";
@@ -237,6 +396,23 @@ static void test_suspend_resume_hpd(chamelium_data_t *data,
 	igt_cleanup_uevents(mon);
 }
 
+/**
+ * SUBTEST: common-hpd-after-suspend
+ * Description: Toggle HPD during suspend on all connectors, check that uevents
+ * 		are sent and connector status is updated
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug, suspend
+ * Mega feature: General Display Features
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: common-hpd-after-hibernate
+ * Description: Toggle HPD during suspend on all connectors, check that uevents
+ *              are sent and connector status is updated
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug, hibernation
+ * Mega feature: General Display Features
+ * Driver requirement: i915, xe
+ */
 static const char test_suspend_resume_hpd_common_desc[] =
 	"Toggle HPD during suspend on all connectors, check that uevents are "
 	"sent and connector status is updated";
@@ -266,6 +442,15 @@ static void test_suspend_resume_hpd_common(chamelium_data_t *data,
 	igt_cleanup_uevents(mon);
 }
 
+/**
+ * SUBTEST: vga-hpd-without-ddc
+ * Description: Disable DDC on a VGA connector, check we still get a uevent on
+ *              hotplug
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: VGA
+ * Driver requirement: i915, xe
+ */
 static const char test_hpd_without_ddc_desc[] =
 	"Disable DDC on a VGA connector, check we still get a uevent on hotplug";
 static void test_hpd_without_ddc(chamelium_data_t *data,
@@ -292,6 +477,25 @@ static void test_hpd_without_ddc(chamelium_data_t *data,
 	igt_cleanup_uevents(mon);
 }
 
+/**
+ * SUBTEST: dp-hpd-storm
+ * Description: Trigger a series of hotplugs in a very small timeframe to
+ *              simulate abad cable, check the kernel falls back to polling
+ *              to avoid a hotplug storm
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: DP
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: hdmi-hpd-storm
+ * Description: Trigger a series of hotplugs in a very small timeframe to
+ *              simulate abad cable, check the kernel falls back to polling
+ *              to avoid a hotplug storm
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: HDMI
+ * Driver requirement: i915, xe
+ */
 static const char test_hpd_storm_detect_desc[] =
 	"Trigger a series of hotplugs in a very small timeframe to simulate a"
 	"bad cable, check the kernel falls back to polling to avoid a hotplug "
@@ -326,6 +530,23 @@ static void test_hpd_storm_detect(chamelium_data_t *data,
 	igt_hpd_storm_reset(data->drm_fd);
 }
 
+/**
+ * SUBTEST: dp-hpd-storm-disable
+ * Description: Disable HPD storm detection, trigger a storm and check the
+ *              kernel doesn't detect one
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: DP
+ * Driver requirement: i915, xe
+ *
+ * SUBTEST: hdmi-hpd-storm-disable
+ * Description: Disable HPD storm detection, trigger a storm and check the
+ *              kernel doesn't detect one
+ * Test category: functionality test
+ * Functionality: chamelium, hotplug
+ * Mega feature: HDMI
+ * Driver requirement: i915, xe
+ */
 static const char test_hpd_storm_disable_desc[] =
 	"Disable HPD storm detection, trigger a storm and check the kernel "
 	"doesn't detect one";
@@ -524,6 +745,6 @@ igt_main
 
 	igt_fixture {
 		igt_display_fini(&data.display);
-		close(data.drm_fd);
+		drm_close_driver(data.drm_fd);
 	}
 }
