@@ -25,7 +25,6 @@
  * Testcase: drmGetMagic() and drmAuthMagic()
  */
 
-#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -43,11 +42,39 @@
 #include <sys/poll.h>
 #include <sys/resource.h>
 #include "drm.h"
+/**
+ * TEST: core auth
+ * Category: Desktop client
+ * Description: Call drmGetMagic() and drmAuthMagic() and see if it behaves.
+ * Feature: core
+ * Functionality: permission management for clients
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: basic-auth
+ * Description: Test magic numbers for master and slave.
+ * Run type: BAT
+ *
+ * SUBTEST: getclient-master-drop
+ * Description: Use 2 clients, check second is authenticated even when first dropped.
+ * Run type: FULL
+ *
+ * SUBTEST: getclient-simple
+ * Description: Check drm client is always authenticated.
+ * Run type: FULL
+ *
+ * SUBTEST: many-magics
+ * Description: Test authentication and magic numbers uniqness for rlimit slaves.
+ * Run type: FULL
+ */
 
 #ifndef __linux__
 # include <pthread.h>
 #endif
 
+#include "igt.h"
+#include "igt_types.h"
+
 IGT_TEST_DESCRIPTION("Call drmGetMagic() and drmAuthMagic() and see if it behaves.");
 
 static bool
@@ -187,13 +214,11 @@ static void test_basic_auth(int master)
 	igt_assert(drmGetMagic(slave, &magic) == 0);
 	igt_assert_eq(magic, old_magic);
 
-	close(slave);
+	drm_close_driver(slave);
 }
 
 igt_main
 {
-	int master;
-
 	/* root (which we run igt as) should always be authenticated */
 	igt_describe("Check drm client is always authenticated.");
 	igt_subtest("getclient-simple") {
@@ -201,7 +226,7 @@ igt_main
 
 		igt_assert(check_auth(fd) == true);
 
-		close(fd);
+		drm_close_driver(fd);
 	}
 
 	igt_describe("Use 2 clients, check second is authenticated even when first dropped.");
@@ -211,15 +236,17 @@ igt_main
 
 		igt_assert(check_auth(fd2) == true);
 
-		close(fd);
+		drm_close_driver(fd);
 
 		igt_assert(check_auth(fd2) == true);
 
-		close(fd2);
+		drm_close_driver(fd2);
 	}
 
 	/* above tests require that no drm fd is open */
 	igt_subtest_group {
+		igt_fd_t(master);
+
 		igt_fixture
 			master = drm_open_driver_master(DRIVER_ANY);
 	
