@@ -39,6 +39,70 @@
 #include "igt_kmod.h"
 #include "igt_sysfs.h"
 #include "sw_sync.h"
+/**
+ * TEST: core hotunplug
+ * Description: Examine behavior of a driver on device hot unplug
+ * Feature: SMI, core
+ * Test category: functionality test
+ *
+ * SUBTEST: hotrebind
+ * Category: Software building block
+ * Sub-category: driver
+ * Functionality: device hotplug
+ * Description:
+ *   Check if the driver can be cleanly rebound to a device with a still open hot unbound driver
+ *   instance
+ * Run type: FULL
+ *
+ * SUBTEST: hotrebind-lateclose
+ * Category: Software building block
+ * Sub-category: driver
+ * Functionality: device hotplug
+ * Description: Check if a hot unbound driver instance still open after hot rebind can be cleanly released
+ * Run type: FULL
+ *
+ * SUBTEST: hotreplug
+ * Category: Software building block
+ * Sub-category: driver
+ * Functionality: device hotplug
+ * Description: Check if a hot unplugged and still open device can be cleanly restored
+ * Run type: FULL
+ *
+ * SUBTEST: hotreplug-lateclose
+ * Category: Software building block
+ * Sub-category: driver
+ * Functionality: device hotplug
+ * Description: Check if an instance of a still open while hot replugged device can be cleanly released
+ * Run type: FULL
+ *
+ * SUBTEST: hotunbind-rebind
+ * Category: Software building block
+ * Sub-category: driver
+ * Functionality: device hotplug
+ * Description: Check if the driver can be cleanly unbound from an open device, then released and rebound
+ * Run type: FULL
+ *
+ * SUBTEST: hotunplug-rescan
+ * Category: Software building block
+ * Sub-category: driver
+ * Functionality: device hotplug
+ * Description: Check if an open device can be cleanly unplugged, then released and restored
+ * Run type: FULL
+ *
+ * SUBTEST: unbind-rebind
+ * Category: Software building block
+ * Sub-category: driver
+ * Functionality: device hotplug
+ * Description: Check if the driver can be cleanly unbound from a device believed to be closed, then rebound
+ * Run type: BAT
+ *
+ * SUBTEST: unplug-rescan
+ * Category: Software building block
+ * Sub-category: driver
+ * Functionality: device hotplug
+ * Description: Check if a device believed to be closed can be cleanly unplugged, then restored
+ * Run type: FULL
+ */
 
 IGT_TEST_DESCRIPTION("Examine behavior of a driver on device hot unplug");
 
@@ -377,7 +441,7 @@ static bool local_i915_perf_healthcheck(int i915)
 {
 	struct intel_perf *intel_perf;
 
-	intel_perf = intel_perf_for_fd(i915);
+	intel_perf = intel_perf_for_fd(i915, 0);
 	if (intel_perf)
 		intel_perf_free(intel_perf);
 	return intel_perf;
@@ -438,6 +502,9 @@ static void node_healthcheck(struct hotunplug *priv, unsigned flags)
 
 static bool healthcheck(struct hotunplug *priv, bool recover)
 {
+	/* give udev some time to recreate device nodes before we continue */
+	sleep(1);
+
 	/* device name may have changed, rebuild IGT device list */
 	igt_devices_scan(true);
 
