@@ -38,6 +38,25 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
+/**
+ * TEST: core setmaster
+ * Category: Desktop client
+ * Description: Check that Drop/SetMaster behaves correctly wrt root/user access
+ * Feature: core
+ * Functionality: permission management for clients
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: master-drop-set-root
+ * Description: Ensure that root can Set/DropMaster
+ *
+ * SUBTEST: master-drop-set-shared-fd
+ * Description: Check the Set/DropMaster behaviour on shared fd
+ *
+ * SUBTEST: master-drop-set-user
+ * Description: Ensure first normal user can Set/DropMaster
+ */
 
 IGT_TEST_DESCRIPTION("Check that Drop/SetMaster behaves correctly wrt root/user"
 		     " access");
@@ -85,7 +104,7 @@ static void check_drop_set(void)
 	igt_assert_eq(drmDropMaster(master), 0);
 	igt_assert_eq(drmSetMaster(master), 0);
 
-	close(master);
+	drm_close_driver(master);
 }
 
 static unsigned tweak_perm(uint8_t *saved_perm, unsigned max_perm, bool save)
@@ -206,10 +225,10 @@ igt_main
 			igt_assert_eq(drmSetMaster(master), -1);
 			igt_assert_eq(errno, EACCES);
 
-			close(master);
+			drm_close_driver(master);
 		}
 		igt_waitchildren();
 
-		close(master);
+		drm_close_driver(master);
 	}
 }
