@@ -25,6 +25,11 @@
  *  Karthik B S <karthik.b.s@intel.com>
  */
 
+/**
+ * TEST: kms async flips
+ * Category: Display
+ * Description: Test asynchronous page flips.
+ */
 #include "igt.h"
 #include "igt_aux.h"
 #include "igt_psr.h"
@@ -41,6 +46,7 @@
 
 #define RUN_TIME 2
 #define MIN_FLIPS_PER_FRAME 5
+#define NUM_FBS 4
 
 IGT_TEST_DESCRIPTION("Test asynchronous page flips.");
 
@@ -48,19 +54,21 @@ typedef struct {
 	int drm_fd;
 	uint32_t crtc_id;
 	uint32_t refresh_rate;
-	struct igt_fb bufs[4];
+	struct igt_fb bufs[NUM_FBS];
 	igt_display_t display;
 	igt_output_t *output;
 	unsigned long flip_timestamp_us;
 	double flip_interval;
+	uint64_t modifier;
+	igt_plane_t *plane;
 	igt_pipe_crc_t *pipe_crc;
 	igt_crc_t ref_crc;
 	int flip_count;
 	int frame_count;
 	bool flip_pending;
-	bool extended;
 	enum pipe pipe;
 	bool alternate_sync_async;
+	bool allow_fail;
 } data_t;
 
 static void flip_handler(int fd_, unsigned int sequence, unsigned int tv_sec,
@@ -115,25 +123,29 @@ static void wait_flip_event(data_t *data)
 	}
 }
 
+static uint64_t default_modifier(data_t *data)
+{
+	if (igt_display_has_format_mod(&data->display, DRM_FORMAT_XRGB8888,
+				       I915_FORMAT_MOD_X_TILED))
+		return I915_FORMAT_MOD_X_TILED;
+	else
+		return DRM_FORMAT_MOD_LINEAR;
+}
+
 static void make_fb(data_t *data, struct igt_fb *fb,
 		    uint32_t width, uint32_t height, int index)
 {
 	int rec_width;
 	cairo_t *cr;
 
-	rec_width = width / (ARRAY_SIZE(data->bufs) * 2);
+	rec_width = width / (NUM_FBS * 2);
 
-	if (is_i915_device(data->drm_fd)) {
-		igt_create_fb(data->drm_fd, width, height, DRM_FORMAT_XRGB8888,
-			      I915_FORMAT_MOD_X_TILED, fb);
-		igt_draw_fill_fb(data->drm_fd, fb, 0x88);
-	} else {
-		igt_create_color_fb(data->drm_fd, width, height, DRM_FORMAT_XRGB8888,
-				    DRM_FORMAT_MOD_LINEAR, 0.0, 0.0, 0.5, fb);
-	}
+	igt_create_color_fb(data->drm_fd, width, height, DRM_FORMAT_XRGB8888,
+			    data->modifier, 0.0, 0.0, 0.5, fb);
 
 	cr = igt_get_cairo_ctx(data->drm_fd, fb);
-	igt_paint_color_rand(cr, rec_width * 2 + rec_width * index, height / 4, rec_width, height / 2);
+	igt_paint_color_rand(cr, rec_width * 2 + rec_width * index, 0, rec_width, height);
+	igt_put_cairo_ctx(cr);
 }
 
 static void require_monotonic_timestamp(int fd)
@@ -144,51 +156,76 @@ static void require_monotonic_timestamp(int fd)
 
 static void test_init(data_t *data)
 {
-	int i;
-	uint32_t width, height;
-	igt_plane_t *plane;
-	static uint32_t prev_output_id;
 	drmModeModeInfo *mode;
 
 	igt_display_reset(&data->display);
 	igt_display_commit(&data->display);
 
 	mode = igt_output_get_mode(data->output);
-	width = mode->hdisplay;
-	height = mode->vdisplay;
 
 	data->crtc_id = data->display.pipes[data->pipe].crtc_id;
 	data->refresh_rate = mode->vrefresh;
 
 	igt_output_set_pipe(data->output, data->pipe);
-	plane = igt_output_get_plane_type(data->output, DRM_PLANE_TYPE_PRIMARY);
 
-	if (prev_output_id != data->output->id) {
+	data->plane = igt_output_get_plane_type(data->output, DRM_PLANE_TYPE_PRIMARY);
+}
+
+static void test_init_fbs(data_t *data)
+{
+	int i;
+	uint32_t width, height;
+	static uint32_t prev_output_id;
+	static uint64_t prev_modifier;
+	drmModeModeInfo *mode;
+
+	mode = igt_output_get_mode(data->output);
+	width = mode->hdisplay;
+	height = mode->vdisplay;
+
+	if (prev_output_id != data->output->id ||
+	    prev_modifier != data->modifier) {
 		prev_output_id = data->output->id;
+		prev_modifier = data->modifier;
 
 		if (data->bufs[0].fb_id) {
-			for (i = 0; i < ARRAY_SIZE(data->bufs); i++)
+			for (i = 0; i < NUM_FBS; i++)
 				igt_remove_fb(data->drm_fd, &data->bufs[i]);
 		}
 
-		for (i = 0; i < ARRAY_SIZE(data->bufs); i++)
+		for (i = 0; i < NUM_FBS; i++)
 			make_fb(data, &data->bufs[i], width, height, i);
 	}
 
-	igt_plane_set_fb(plane, &data->bufs[0]);
-	igt_plane_set_size(plane, width, height);
+	igt_plane_set_fb(data->plane, &data->bufs[0]);
+	igt_plane_set_size(data->plane, width, height);
 
 	igt_display_commit2(&data->display, data->display.is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 }
 
+/**
+ * SUBTEST: alternate-sync-async-flip
+ * Description: Verify the async flip functionality and the fps during async flips
+ *              Alternate between sync and async flips
+ * Driver requirement: i915, xe
+ * Functionality: async_flips, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: async-flip-with-page-flip-events
+ * Description: Verify the async flip functionality and the fps during async flips
+ *              Wait for page flip events in between successive asynchronous flips
+ * Driver requirement: i915, xe
+ * Functionality: async_flips, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void test_async_flip(data_t *data)
 {
 	int ret, frame;
 	long long int fps;
 	struct timeval start, end, diff;
 
-	test_init(data);
-
 	gettimeofday(&start, NULL);
 	frame = 1;
 	do {
@@ -198,7 +235,7 @@ static void test_async_flip(data_t *data)
 			flags &= ~DRM_MODE_PAGE_FLIP_ASYNC;
 
 			ret = drmModePageFlip(data->drm_fd, data->crtc_id,
-					      data->bufs[frame % 4].fb_id,
+					      data->bufs[frame % NUM_FBS].fb_id,
 					      flags, data);
 
 			igt_assert(ret == 0);
@@ -215,12 +252,12 @@ static void test_async_flip(data_t *data)
 			 * in order to change the watermark levels as per the optimization. Hence the
 			 * subsequent async flips will actually do the asynchronous flips.
 			 */
-			if (is_i915_device(data->drm_fd)) {
+			if (is_intel_device(data->drm_fd)) {
 				uint32_t devid = intel_get_drm_devid(data->drm_fd);
 
 				if (IS_GEN9(devid) || IS_GEN10(devid) || AT_LEAST_GEN(devid, 12)) {
 					ret = drmModePageFlip(data->drm_fd, data->crtc_id,
-							      data->bufs[frame % 4].fb_id,
+							      data->bufs[frame % NUM_FBS].fb_id,
 							      flags, data);
 
 					igt_assert(ret == 0);
@@ -231,10 +268,12 @@ static void test_async_flip(data_t *data)
 		}
 
 		ret = drmModePageFlip(data->drm_fd, data->crtc_id,
-				      data->bufs[frame % 4].fb_id,
+				      data->bufs[frame % NUM_FBS].fb_id,
 				      flags, data);
-
-		igt_assert(ret == 0);
+		if (frame == 1 && data->allow_fail)
+			igt_skip_on(ret == -EINVAL);
+		else
+			igt_assert(ret == 0);
 
 		wait_flip_event(data);
 
@@ -276,6 +315,16 @@ static void wait_for_vblank(data_t *data, unsigned long *vbl_time, unsigned int
 	*seq = wait_vbl.reply.sequence;
 }
 
+/**
+ * SUBTEST: test-time-stamp
+ * Description: Verify the async flip functionality and the fps during async flips
+ *              Verify that the async flip timestamp does not coincide with either
+ *              previous or next vblank
+ * Driver requirement: i915, xe
+ * Functionality: async_flips, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void test_timestamp(data_t *data)
 {
 	int flags = DRM_MODE_PAGE_FLIP_ASYNC | DRM_MODE_PAGE_FLIP_EVENT;
@@ -283,8 +332,6 @@ static void test_timestamp(data_t *data)
 	unsigned int seq, seq1;
 	int ret;
 
-	test_init(data);
-
 	/*
 	 * In older platforms(<= gen10), async address update bit is double buffered.
 	 * So flip timestamp can be verified only from the second flip.
@@ -322,6 +369,14 @@ static void test_timestamp(data_t *data)
 		     "Async flip time stamp is expected to be in between 2 vblank time stamps\n");
 }
 
+/**
+ * SUBTEST: test-cursor
+ * Description: Verify that the DRM_IOCTL_MODE_CURSOR passes after async flip
+ * Driver requirement: i915, xe
+ * Functionality: async_flips, cursor, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void test_cursor(data_t *data)
 {
 	int flags = DRM_MODE_PAGE_FLIP_ASYNC | DRM_MODE_PAGE_FLIP_EVENT;
@@ -342,8 +397,6 @@ static void test_cursor(data_t *data)
 	do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_WIDTH, &width));
 	do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_WIDTH, &height));
 
-	test_init(data);
-
 	igt_create_color_fb(data->drm_fd, width, height, DRM_FORMAT_ARGB8888,
 			    DRM_FORMAT_MOD_LINEAR, 1., 1., 1., &cursor_fb);
 
@@ -384,8 +437,6 @@ static void test_invalid(data_t *data)
 	width = mode->hdisplay;
 	height = mode->vdisplay;
 
-	test_init(data);
-
 	igt_create_fb(data->drm_fd, width, height, DRM_FORMAT_XRGB8888,
 		      I915_FORMAT_MOD_Y_TILED, &fb);
 
@@ -478,6 +529,14 @@ static unsigned int clock_ms(void)
 	return ts.tv_sec * 1000 + ts.tv_nsec / 1000000;
 }
 
+/**
+ * SUBTEST: crc
+ * Description: Use CRC to verify async flip scans out the correct framebuffer
+ * Driver requirement: i915, xe
+ * Functionality: async_flips, crc, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void test_crc(data_t *data)
 {
 	unsigned int frame = 0;
@@ -489,13 +548,13 @@ static void test_crc(data_t *data)
 	data->frame_count = 0;
 	data->flip_pending = false;
 
-	test_init(data);
-
 	cr = igt_get_cairo_ctx(data->drm_fd, &data->bufs[frame]);
 	igt_paint_color(cr, 0, 0, data->bufs[frame].width, data->bufs[frame].height, 1.0, 0.0, 0.0);
+	igt_put_cairo_ctx(cr);
 
 	cr = igt_get_cairo_ctx(data->drm_fd, &data->bufs[!frame]);
 	igt_paint_color(cr, 0, 0, data->bufs[!frame].width, data->bufs[!frame].height, 1.0, 0.0, 0.0);
+	igt_put_cairo_ctx(cr);
 
 	ret = drmModeSetCrtc(data->drm_fd, data->crtc_id, data->bufs[frame].fb_id, 0, 0,
 			     &data->output->config.connector->connector_id, 1,
@@ -517,6 +576,7 @@ static void test_crc(data_t *data)
 		/* fill the next fb with the expected color */
 		cr = igt_get_cairo_ctx(data->drm_fd, &data->bufs[frame]);
 		igt_paint_color(cr, 0, 0, 1, data->bufs[frame].height, 1.0, 0.0, 0.0);
+		igt_put_cairo_ctx(cr);
 
 		data->flip_pending = true;
 		ret = drmModePageFlip(data->drm_fd, data->crtc_id, data->bufs[frame].fb_id,
@@ -529,6 +589,7 @@ static void test_crc(data_t *data)
 		frame = !frame;
 		cr = igt_get_cairo_ctx(data->drm_fd, &data->bufs[frame]);
 		igt_paint_color_rand(cr, 0, 0, 1, data->bufs[frame].height);
+		igt_put_cairo_ctx(cr);
 	}
 
 	igt_pipe_crc_stop(data->pipe_crc);
@@ -538,44 +599,60 @@ static void test_crc(data_t *data)
 	igt_assert_lt(data->frame_count * 2, data->flip_count);
 }
 
+/**
+ * SUBTEST: invalid-async-flip
+ * Description: Negative case to verify if changes in fb are rejected from kernel as expected
+ * Driver requirement: i915, xe
+ * Functionality: async_flips, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void run_test(data_t *data, void (*test)(data_t *))
 {
-	igt_output_t *output;
-	enum pipe pipe;
+	igt_display_t *display = &data->display;
 
-	for_each_pipe(&data->display, pipe) {
-		for_each_valid_output_on_pipe(&data->display, pipe, output) {
-			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), output->name) {
-				data->output = output;
-				data->pipe = pipe;
-				test(data);
-			}
+	for_each_pipe_with_valid_output(display, data->pipe, data->output) {
+		igt_display_reset(display);
 
-			if (!data->extended)
-				break;
+		igt_output_set_pipe(data->output, data->pipe);
+		if (!i915_pipe_output_combo_valid(display))
+			continue;
+
+		test_init(data);
+		data->allow_fail = false;
+		data->modifier = default_modifier(data);
+		igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(data->pipe), data->output->name) {
+			test_init_fbs(data);
+			test(data);
 		}
 	}
 }
 
-static int opt_handler(int opt, int opt_index, void *_data)
+static void run_test_with_modifiers(data_t *data, void (*test)(data_t *))
 {
-	data_t *data = _data;
+	for_each_pipe_with_valid_output(&data->display, data->pipe, data->output) {
+		test_init(data);
 
-	switch (opt) {
-	case 'e':
-		data->extended = true;
-		break;
+		for (int i = 0; i < data->plane->format_mod_count; i++) {
+			if (data->plane->formats[i] != DRM_FORMAT_XRGB8888)
+				continue;
+
+			data->allow_fail = true;
+			data->modifier = data->plane->modifiers[i];
+
+			igt_dynamic_f("pipe-%s-%s-%s", kmstest_pipe_name(data->pipe),
+				      data->output->name,
+				      igt_fb_modifier_name(data->modifier)) {
+				test_init_fbs(data);
+				test(data);
+			}
+		}
 	}
-
-	return IGT_OPT_HANDLER_SUCCESS;
 }
 
-static const char help_str[] =
-	"  --e \t\tRun the extended tests\n";
-
 static data_t data;
 
-igt_main_args("e", NULL, help_str, opt_handler, &data)
+igt_main
 {
 	int i;
 
@@ -597,7 +674,7 @@ igt_main_args("e", NULL, help_str, opt_handler, &data)
 		igt_describe("Wait for page flip events in between successive asynchronous flips");
 		igt_subtest_with_dynamic("async-flip-with-page-flip-events") {
 			data.alternate_sync_async = false;
-			run_test(&data, test_async_flip);
+			run_test_with_modifiers(&data, test_async_flip);
 		}
 
 		igt_describe("Alternate between sync and async flips");
@@ -628,7 +705,7 @@ igt_main_args("e", NULL, help_str, opt_handler, &data)
 	igt_describe("Negative case to verify if changes in fb are rejected from kernel as expected");
 	igt_subtest_with_dynamic("invalid-async-flip") {
 		/* TODO: support more vendors */
-		igt_require(is_i915_device(data.drm_fd));
+		igt_require(is_intel_device(data.drm_fd));
 		igt_require(igt_display_has_format_mod(&data.display, DRM_FORMAT_XRGB8888,
 						       I915_FORMAT_MOD_Y_TILED));
 
@@ -644,12 +721,12 @@ igt_main_args("e", NULL, help_str, opt_handler, &data)
 	}
 
 	igt_fixture {
-		for (i = 0; i < ARRAY_SIZE(data.bufs); i++)
+		for (i = 0; i < NUM_FBS; i++)
 			igt_remove_fb(data.drm_fd, &data.bufs[i]);
 
 		igt_display_reset(&data.display);
 		igt_display_commit(&data.display);
 		igt_display_fini(&data.display);
-		close(data.drm_fd);
+		drm_close_driver(data.drm_fd);
 	}
 }
