@@ -27,8 +27,10 @@
  *    Pekka Paalanen <pekka.paalanen@collabora.co.uk>
  */
 
-/*
- * Testcase: testing atomic modesetting API
+/**
+ * TEST: kms atomic
+ * Category: Display
+ * Description: Test atomic modesetting API
  */
 
 #include <unistd.h>
@@ -287,6 +289,15 @@ static uint32_t plane_get_igt_format(igt_plane_t *plane)
 	return 0;
 }
 
+/**
+ * SUBTEST: plane-primary-overlay-mutable-zpos
+ * Description: Verify that the overlay plane can cover the primary one (and
+ *              vice versa) by changing their zpos property.
+ * Driver requirement: i915, xe
+ * Functionality: kms_core, plane
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void
 plane_primary_overlay_mutable_zpos(igt_pipe_t *pipe, igt_output_t *output,
 				   igt_plane_t *primary, igt_plane_t *overlay,
@@ -305,12 +316,12 @@ plane_primary_overlay_mutable_zpos(igt_pipe_t *pipe, igt_output_t *output,
 	uint32_t h_overlay = mode->vdisplay / 2;
 
 	igt_create_color_pattern_fb(pipe->display->drm_fd,
-				    w, h, format_primary, I915_TILING_NONE,
+				    w, h, format_primary, DRM_FORMAT_MOD_LINEAR,
 				    0.2, 0.2, 0.2, &fb_primary);
 
 	igt_create_color_pattern_fb(pipe->display->drm_fd,
 				    w_overlay, h_overlay,
-				    format_overlay, I915_TILING_NONE,
+				    format_overlay, DRM_FORMAT_MOD_LINEAR,
 				    0.2, 0.2, 0.2, &fb_overlay);
 
 	/* Draw a hole in the overlay */
@@ -369,6 +380,15 @@ plane_primary_overlay_mutable_zpos(igt_pipe_t *pipe, igt_output_t *output,
 	igt_assert_eq_u64(igt_plane_get_prop(overlay, IGT_PLANE_ZPOS), 1);
 }
 
+/**
+ * SUBTEST: plane-immutable-zpos
+ * Description: Verify the reported zpos property of planes by making sure only
+ *              higher zpos planes cover the lower zpos ones.
+ * Driver requirement: i915, xe
+ * Functionality: kms_core, plane
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void
 plane_immutable_zpos(igt_display_t *display, igt_pipe_t *pipe,
 		     igt_output_t *output)
@@ -404,7 +424,7 @@ plane_immutable_zpos(igt_display_t *display, igt_pipe_t *pipe,
 	igt_create_color_fb(display->drm_fd,
 			    w_lower, h_lower,
 			    DRM_FORMAT_XRGB8888,
-			    I915_TILING_NONE,
+			    DRM_FORMAT_MOD_LINEAR,
 			    0.0, 0.0, 0.0, &fb_ref);
 
 	/* create reference image */
@@ -436,7 +456,6 @@ plane_immutable_zpos(igt_display_t *display, igt_pipe_t *pipe,
 			continue;
 
 		zpos = igt_plane_get_prop(temp, IGT_PLANE_ZPOS);
-		igt_assert_lt(zpos, n_planes);
 
 		plane_ptr[zpos] = temp;
 	}
@@ -444,14 +463,14 @@ plane_immutable_zpos(igt_display_t *display, igt_pipe_t *pipe,
 	fb_id_lower = igt_create_color_fb(display->drm_fd,
 					  w_lower, h_lower,
 					  DRM_FORMAT_XRGB8888,
-					  I915_TILING_NONE,
+					  DRM_FORMAT_MOD_LINEAR,
 					  0.0, 0.0, 1.0, &fb_lower);
 	igt_assert(fb_id_lower);
 
 	fb_id_upper = igt_create_color_fb(display->drm_fd,
 					  w_upper, h_upper,
 					  DRM_FORMAT_XRGB8888,
-					  I915_TILING_NONE,
+					  DRM_FORMAT_MOD_LINEAR,
 					  1.0, 1.0, 0.0, &fb_upper);
 	igt_assert(fb_id_upper);
 
@@ -505,6 +524,21 @@ plane_immutable_zpos(igt_display_t *display, igt_pipe_t *pipe,
 	igt_remove_fb(display->drm_fd, &fb_upper);
 }
 
+/**
+ * SUBTEST: plane-%s-legacy
+ * Description: Test for KMS atomic modesetting on %arg[1] and ensure coherency
+ *              between legacy and atomic interfaces.
+ * Driver requirement: i915, xe
+ * Functionality: kms_core, plane
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @cursor:         Cursor plane
+ * @overlay:        Overlay plane
+ * @primary:        Primary plane
+ */
 static void plane_overlay(igt_pipe_t *pipe, igt_output_t *output, igt_plane_t *plane)
 {
 	drmModeModeInfo *mode = igt_output_get_mode(output);
@@ -516,7 +550,7 @@ static void plane_overlay(igt_pipe_t *pipe, igt_output_t *output, igt_plane_t *p
 	igt_require(format != 0);
 
 	igt_create_pattern_fb(pipe->display->drm_fd, w, h,
-			      format, I915_TILING_NONE, &fb);
+			      format, DRM_FORMAT_MOD_LINEAR, &fb);
 
 	igt_plane_set_fb(plane, &fb);
 	igt_plane_set_position(plane, w/2, h/2);
@@ -551,7 +585,7 @@ static void plane_primary(igt_pipe_t *pipe, igt_plane_t *plane, struct igt_fb *f
 
 	igt_create_color_pattern_fb(pipe->display->drm_fd,
 				    fb->width, fb->height,
-				    fb->drm_format, I915_TILING_NONE,
+				    fb->drm_format, DRM_FORMAT_MOD_LINEAR,
 				    0.2, 0.2, 0.2, &fb2);
 
 	/* Flip the primary plane using the atomic API, and double-check
@@ -579,6 +613,16 @@ static void plane_primary(igt_pipe_t *pipe, igt_plane_t *plane, struct igt_fb *f
 	plane_commit(plane, COMMIT_UNIVERSAL, ATOMIC_RELAX_NONE);
 }
 
+/**
+ * SUBTEST: test-only
+ * Description: Test to ensure that DRM_MODE_ATOMIC_TEST_ONLY really only touches
+ *              the free-standing state objects and nothing else.
+ * Driver requirement: i915, xe
+ * Functionality: kms_core
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
 /* test to ensure that DRM_MODE_ATOMIC_TEST_ONLY really only touches the
  * free-standing state objects and nothing else.
  */
@@ -600,7 +644,7 @@ static void test_only(igt_pipe_t *pipe_obj,
 
 	igt_create_pattern_fb(pipe_obj->display->drm_fd,
 			     mode->hdisplay, mode->vdisplay,
-			     format, I915_TILING_NONE, &fb);
+			     format, DRM_FORMAT_MOD_LINEAR, &fb);
 	igt_plane_set_fb(primary, &fb);
 	igt_output_set_pipe(output, pipe_obj->pipe);
 
@@ -687,6 +731,19 @@ static void plane_cursor(igt_pipe_t *pipe_obj,
 	plane_commit(cursor, COMMIT_LEGACY, ATOMIC_RELAX_NONE);
 }
 
+/**
+ * SUBTEST: plane-invalid-%s
+ * Description: Test error handling when invalid %arg[1] are passed
+ * Driver requirement: i915, xe
+ * Functionality: kms_core, plane
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @params:           plane parameters
+ * @params-fence:     plane fence parameters
+ */
 static void plane_invalid_params(igt_pipe_t *pipe,
 				 igt_output_t *output,
 				 igt_plane_t *plane,
@@ -737,7 +794,7 @@ static void plane_invalid_params(igt_pipe_t *pipe,
 	/* Create a framebuffer too small for the plane configuration. */
 	igt_create_pattern_fb(pipe->display->drm_fd,
 			      fb->width - 1, fb->height - 1,
-			      fb->drm_format, I915_TILING_NONE, &fb2);
+			      fb->drm_format, DRM_FORMAT_MOD_LINEAR, &fb2);
 
 	igt_plane_set_prop_value(plane, IGT_PLANE_FB_ID, fb2.fb_id);
 	plane_commit_atomic_err(plane, ATOMIC_RELAX_NONE, ENOSPC);
@@ -776,6 +833,19 @@ static void plane_invalid_params_fence(igt_pipe_t *pipe,
 	close(timeline);
 }
 
+/**
+ * SUBTEST: crtc-invalid-%s
+ * Description: Test error handling when invalid %arg[1] are passed
+ * Driver requirement: i915, xe
+ * Functionality: kms_core
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @params:           crtc parameters
+ * @params-fence:     crtc fence parameters
+ */
 static void crtc_invalid_params(igt_pipe_t *pipe,
 				igt_output_t *output,
 				igt_plane_t *plane,
@@ -923,6 +993,17 @@ static void crtc_invalid_params_fence(igt_pipe_t *pipe,
 	igt_assert(pipe->out_fence_fd != -1);
 }
 
+/**
+ * SUBTEST: atomic-invalid-params
+ * Description: Test abuse the atomic ioctl directly in order to test various
+ *              invalid conditions which the libdrm wrapper won't allow us to
+ *              create.
+ * Driver requirement: i915, xe
+ * Functionality: kms_core
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
 /* Abuse the atomic ioctl directly in order to test various invalid conditions,
  * which the libdrm wrapper won't allow us to create. */
 static void atomic_invalid_params(igt_pipe_t *pipe,
@@ -1064,6 +1145,14 @@ static void atomic_invalid_params(igt_pipe_t *pipe,
 	do_ioctl_err(display->drm_fd, DRM_IOCTL_MODE_ATOMIC, &ioc, EFAULT);
 }
 
+/**
+ * SUBTEST: atomic_plane_damage
+ * Description: Simple test cases to use FB_DAMAGE_CLIPS plane property
+ * Driver requirement: i915, xe
+ * Functionality: kms_core, plane
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void atomic_plane_damage(igt_pipe_t *pipe, igt_plane_t *plane, struct igt_fb *fb)
 {
 	struct drm_mode_rect damage[2];
@@ -1072,7 +1161,7 @@ static void atomic_plane_damage(igt_pipe_t *pipe, igt_plane_t *plane, struct igt
 
 	/* Color fb with white rect at center */
 	igt_create_color_fb(pipe->display->drm_fd, fb->width, fb->height,
-			    fb->drm_format, I915_TILING_NONE, 0.2, 0.2, 0.2,
+			    fb->drm_format, DRM_FORMAT_MOD_LINEAR, 0.2, 0.2, 0.2,
 			    &fb_1);
 	cr_1 = igt_get_cairo_ctx(pipe->display->drm_fd, &fb_1);
 	igt_paint_color(cr_1, fb->width/4, fb->height/4, fb->width/2,
@@ -1111,7 +1200,7 @@ static void atomic_plane_damage(igt_pipe_t *pipe, igt_plane_t *plane, struct igt
 	 * issue plane update with damage and verify the state.
 	 */
 	igt_create_color_fb(pipe->display->drm_fd, fb->width, fb->height,
-			    fb->drm_format, I915_TILING_NONE, 0.2, 0.2, 0.2,
+			    fb->drm_format, DRM_FORMAT_MOD_LINEAR, 0.2, 0.2, 0.2,
 			    &fb_2);
 
 	damage[0].x1 = fb->width/2;
@@ -1141,7 +1230,7 @@ static void atomic_plane_damage(igt_pipe_t *pipe, igt_plane_t *plane, struct igt
 	/* Reszie fb_1 to be bigger than plane */
 	igt_remove_fb(pipe->display->drm_fd, &fb_1);
 	igt_create_color_fb(pipe->display->drm_fd, fb->width * 2, fb->height,
-			    fb->drm_format, I915_TILING_NONE, 0.2, 0.2, 0.2,
+			    fb->drm_format, DRM_FORMAT_MOD_LINEAR, 0.2, 0.2, 0.2,
 			    &fb_1);
 
 	damage[0].x1 = fb->width;
@@ -1265,6 +1354,10 @@ static void atomic_plane_damage(igt_pipe_t *pipe, igt_plane_t *plane, struct igt
 
 static void atomic_setup(igt_display_t *display, enum pipe pipe, igt_output_t *output, igt_plane_t *primary, struct igt_fb *fb)
 {
+	igt_info("Using (pipe %s + %s) to run the subtest.\n",
+		 kmstest_pipe_name(pipe), igt_output_name(output));
+
+	igt_display_reset(display);
 	igt_output_set_pipe(output, pipe);
 	igt_plane_set_fb(primary, fb);
 
@@ -1293,6 +1386,7 @@ igt_main
 	igt_plane_t *primary = NULL;
 	drmModeModeInfo *mode;
 	struct igt_fb fb;
+	bool valid_config = false;
 
 	igt_fixture {
 		display.drm_fd = drm_open_driver_master(DRIVER_ANY);
@@ -1303,8 +1397,16 @@ igt_main
 		igt_require(display.is_atomic);
 		igt_display_require_output(&display);
 
-		for_each_pipe_with_valid_output(&display, pipe, output)
-			break;
+		for_each_pipe_with_valid_output(&display, pipe, output) {
+			igt_display_reset(&display);
+
+			igt_output_set_pipe(output, pipe);
+			if (i915_pipe_output_combo_valid(&display)) {
+				valid_config = true;
+				break;
+			}
+		}
+		igt_require(valid_config);
 
 		pipe_obj = &display.pipes[pipe];
 		primary = igt_pipe_get_plane_type(pipe_obj, DRM_PLANE_TYPE_PRIMARY);
@@ -1342,9 +1444,13 @@ igt_main
 	igt_subtest("plane-primary-overlay-mutable-zpos") {
 		uint32_t format_primary = DRM_FORMAT_ARGB8888;
 		uint32_t format_overlay = DRM_FORMAT_ARGB1555;
+		igt_plane_t *overlay;
 
-		igt_plane_t *overlay =
-			igt_pipe_get_plane_type(pipe_obj, DRM_PLANE_TYPE_OVERLAY);
+		igt_info("Using (pipe %s + %s) to run the subtest.\n",
+			 kmstest_pipe_name(pipe), igt_output_name(output));
+
+		igt_display_reset(&display);
+		overlay = igt_pipe_get_plane_type(pipe_obj, DRM_PLANE_TYPE_OVERLAY);
 		igt_require(overlay);
 
 		igt_require(igt_plane_has_prop(primary, IGT_PLANE_ZPOS));
@@ -1361,6 +1467,10 @@ igt_main
 	igt_describe("Verify the reported zpos property of planes by making sure "\
 		     "only higher zpos planes cover the lower zpos ones.");
 	igt_subtest("plane-immutable-zpos") {
+		igt_info("Using (pipe %s + %s) to run the subtest.\n",
+			 kmstest_pipe_name(pipe), igt_output_name(output));
+
+		igt_display_reset(&display);
 		igt_output_set_pipe(output, pipe);
 		plane_immutable_zpos(&display, pipe_obj, output);
 	}
@@ -1370,6 +1480,9 @@ igt_main
 	igt_subtest("test-only") {
 		atomic_clear(&display, pipe, primary, output);
 
+		igt_info("Using (pipe %s + %s) to run the subtest.\n",
+			 kmstest_pipe_name(pipe), igt_output_name(output));
+
 		test_only(pipe_obj, primary, output);
 	}
 
@@ -1436,6 +1549,6 @@ igt_main
 		igt_remove_fb(display.drm_fd, &fb);
 
 		igt_display_fini(&display);
-		close(display.drm_fd);
+		drm_close_driver(display.drm_fd);
 	}
 }
