@@ -28,6 +28,32 @@
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
+/**
+ * TEST: kms cursor edge walk
+ * Category: Display
+ * Description: Exercise CHV pipe C cursor fail
+ * Description: Test to check different cursor sizes by walking different edges of screen
+ *
+ * SUBTEST: %s-%s
+ * Description: Checking cursor size %arg[1] by walking %arg[2] of screen
+ * Driver requirement: i915, xe
+ * Functionality: cursor
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @128x128:          128x128
+ * @256x256:          256x256
+ * @64x64:            64x64
+ *
+ * arg[2]:
+ *
+ * @left-edge:        Left edge
+ * @right-edge:       Right edge
+ * @top-bottom:       Top to bottom
+ * @top-edge:         Top edge
+ */
 
 IGT_TEST_DESCRIPTION("Exercise CHV pipe C cursor fail");
 
@@ -60,6 +86,10 @@ enum {
 	EDGE_BOTTOM = 0x8,
 };
 
+static bool extended;
+static enum pipe active_pipes[IGT_MAX_PIPES];
+static uint32_t last_pipe;
+
 static void create_cursor_fb(data_t *data, int cur_w, int cur_h)
 {
 	cairo_t *cr;
@@ -284,6 +314,9 @@ static int opt_handler(int opt, int opt_index, void *_data)
 	case 'd':
 		data->disable = true;
 		break;
+	case 'e':
+		extended = true;
+		break;
 	case 'j':
 		data->jump = true;
 		break;
@@ -300,12 +333,14 @@ static const struct option long_opts[] = {
 	{ .name = "colored", .val = 'c' },
 	{ .name = "disable", .val = 'd'},
 	{ .name = "jump", .val = 'j' },
+	{ .name = "extended", .val = 'e'},
 	{}
 };
 static const char *help_str =
 	"  --colored\t\tUse a colored cursor (disables CRC checks)\n"
 	"  --disable\t\tDisable the cursor between each step\n"
-	"  --jump\t\tJump the cursor to middle of the screen between each step)\n";
+	"  --jump\t\tJump the cursor to middle of the screen between each step)\n"
+	"  --extended\t\tRun on all pipes.(Default it will Run only two pipes)\n";
 
 igt_main_args("", long_opts, help_str, opt_handler, &data)
 {
@@ -322,10 +357,11 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 
 	igt_fixture {
 		int ret;
+		enum pipe pipe;
 
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 
-		if (is_i915_device(data.drm_fd))
+		if (is_intel_device(data.drm_fd))
 			data.devid = intel_get_drm_devid(data.drm_fd);
 
 		ret = drmGetCap(data.drm_fd, DRM_CAP_CURSOR_WIDTH, &max_curw);
@@ -340,23 +376,40 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 
 		igt_display_require(&data.display, data.drm_fd);
 		igt_display_require_output(&data.display);
+
+		/* Get active pipes. */
+		last_pipe = 0;
+		for_each_pipe(&data.display, pipe)
+			active_pipes[last_pipe++] = pipe;
+		last_pipe--;
 	}
 
-	igt_describe("Checking cursor by walking left/right/top/bottom edge of screen");
-	igt_subtest_group {
-		for (i = 0; i < ARRAY_SIZE(tests); i++) {
-			igt_subtest_with_dynamic(tests[i].name) {
-				for_each_pipe_with_single_output(&data.display, data.pipe, data.output) {
-					for (data.curw = 64; data.curw <= 256; data.curw *= 2) {
-						data.curh = data.curw;
-						igt_require(data.curw <= max_curw && data.curh <= max_curh);
+	for (i = 0; i < ARRAY_SIZE(tests); i++) {
+		for (data.curw = 64; data.curw <= 256; data.curw *= 2) {
+			data.curh = data.curw;
+			igt_fixture
+				igt_require(data.curw <= max_curw && data.curh <= max_curh);
 
-						igt_dynamic_f("pipe-%s-%s-%dx%d",
-							      kmstest_pipe_name(data.pipe),
-							      data.output->name,
-							      data.curw, data.curh)
-							test_crtc(&data, tests[i].flags);
-					}
+			igt_describe_f("Checking cursor size %dx%d by walking %s of screen",
+					data.curw, data.curh, tests[i].name);
+			igt_subtest_with_dynamic_f("%dx%d-%s", data.curw,
+						   data.curh, tests[i].name) {
+				for_each_pipe_with_single_output(&data.display, data.pipe, data.output) {
+					if (!extended && data.pipe != active_pipes[0] &&
+					    data.pipe != active_pipes[last_pipe])
+						continue;
+
+					igt_display_reset(&data.display);
+					igt_output_set_pipe(data.output, data.pipe);
+					if (!i915_pipe_output_combo_valid(&data.display))
+						continue;
+
+					igt_output_set_pipe(data.output, PIPE_NONE);
+
+					igt_dynamic_f("pipe-%s-%s",
+						      kmstest_pipe_name(data.pipe),
+						      data.output->name)
+						test_crtc(&data, tests[i].flags);
 				}
 			}
 		}
@@ -364,6 +417,6 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 
 	igt_fixture {
 		igt_display_fini(&data.display);
-		close(data.drm_fd);
+		drm_close_driver(data.drm_fd);
 	}
 }
