@@ -31,6 +31,18 @@
 
 #include "igt.h"
 #include "igt_kms.h"
+/**
+ * TEST: kms dp aux dev
+ * Category: Display
+ * Description: Test that /dev/drm_dp_aux reads work
+ *
+ * SUBTEST:
+ * Description: Test that /dev/drm_dp_aux reads work
+ * Driver requirement: i915, xe
+ * Functionality: dp_aux
+ * Mega feature: DP
+ * Test category: functionality test
+ */
 
 IGT_TEST_DESCRIPTION("Test that /dev/drm_dp_aux reads work");
 
@@ -42,6 +54,20 @@ static bool is_mst_connector(int drm_fd, uint32_t connector_id)
 				    NULL, NULL);
 }
 
+static bool sink_detect_error(int drm_fd, uint32_t connector_id, int error_code)
+{
+	switch (error_code) {
+	case ETIMEDOUT:
+	case ENXIO:
+		return true;
+	case EIO:
+		return is_mst_connector(drm_fd, connector_id) ||
+		       is_amdgpu_device(drm_fd);
+	default:
+		return false;
+	};
+}
+
 static bool test(int drm_fd, uint32_t connector_id)
 {
 	drmModeConnector *connector;
@@ -85,9 +111,7 @@ static bool test(int drm_fd, uint32_t connector_id)
 			 ret > 0 ? "success" : strerror(errno));
 
 		igt_assert(ret == sizeof(buf) ||
-			   errno == ETIMEDOUT ||
-			   (errno == EIO && (is_mst_connector(drm_fd, connector_id) ||
-			    is_amdgpu_device(drm_fd))));
+			   sink_detect_error(drm_fd, connector_id, errno));
 
 		close(fd);
 
@@ -135,5 +159,5 @@ igt_simple_main
 	igt_require(valid_connectors);
 
 	drmModeFreeResources(res);
-	close(drm_fd);
+	drm_close_driver(drm_fd);
 }
