@@ -26,6 +26,11 @@
  * This is a test of DRM leases
  */
 
+/**
+ * TEST: kms lease
+ * Category: Display
+ * Description: Test of CreateLease.
+ */
 
 #include "igt.h"
 #include <stdlib.h>
@@ -45,6 +50,160 @@
 #include <drm.h>
 #include "igt_device.h"
 
+/**
+ * SUBTEST: atomic_implicit_crtc
+ * Description: Negative test by using a different crtc with atomic ioctl
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: cursor_implicit_plane
+ * Description: Negative test by using a non-primary plane with setcursor ioctl
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: empty_lease
+ * Description: Check that creating an empty lease works
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: implicit-plane-lease
+ * Description: Tests the implicitly added planes.
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: invalid-create-leases
+ * Description: Tests error handling while creating invalid corner-cases for
+ *              create-lease ioctl
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lease-uevent
+ * Description: Tests all the uevent cases
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lease_again
+ * Description: Tests leasing objects more than once
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lease_get
+ * Description: Tests getting the required contents of a lease
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lease_invalid_connector
+ * Description: Tests leasing an invalid connector
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lease_invalid_crtc
+ * Description: Tests leasing an invalid crtc
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lease_invalid_plane
+ * Description: Tests leasing an invalid plane
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lease_revoke
+ * Description: Tests revocation of lease
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lease_unleased_connector
+ * Description: Negative test by trying to use an unleased connector
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lease_unleased_crtc
+ * Description: Negative test by trying to use an unleased crtc
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: lessee_list
+ * Description: Check if listed lease is same as created one
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: master-vs-lease
+ * Description: Tests the drop/set_master interactions.
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: multimaster-lease
+ * Description: Tests that the 2nd master can only create leases while being
+ *              active master, and that leases on the first master don't prevent
+ *              lease creation for the 2nd master.
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: page_flip_implicit_plane
+ * Description: Negative test by using a non-primary plane with the page flip ioctl
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: possible-crtcs-filtering
+ * Description: Tests that  possible_crtcs logically match between master and
+ *              lease, and that the values are correctly renumbered on the lease
+ *              side.
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: setcrtc_implicit_plane
+ * Description: Negative test by using a non-primary plane with the setcrtc ioctl
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: simple_lease
+ * Description: Check if create lease ioctl call works
+ * Driver requirement: i915, xe
+ * Functionality: lease
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
 IGT_TEST_DESCRIPTION("Test of CreateLease.");
 
 typedef struct {
@@ -77,7 +236,7 @@ static int prepare_crtc(lease_t *lease, data_t *data)
 	drmModeModeInfo *mode;
 	igt_display_t *display = &lease->display;
 	igt_output_t *output = connector_id_to_output(display, data->connector_id);
-	enum pipe pipe = data->pipe;
+	enum pipe pipe = display->pipes[data->pipe].pipe;
 	igt_plane_t *primary;
 	int ret;
 
@@ -207,7 +366,7 @@ static int paint_fb(int drm_fd, struct igt_fb *fb, const char *test_name,
 	igt_cairo_printf_line(cr, align_hcenter, 10, "%s", connector_str);
 	igt_cairo_printf_line(cr, align_hcenter, 10, "%s", pipe_str);
 
-	cairo_destroy(cr);
+	igt_put_cairo_ctx(cr);
 
 	return 0;
 }
@@ -317,6 +476,7 @@ static void setcrtc_implicit_plane(data_t *data)
 	struct drm_mode_create_lease mcl;
 	drmModePlaneRes *plane_resources;
 	uint32_t wrong_plane_id = 0;
+	int ret = 0, ret_mcl = 0;
 	igt_output_t *output =
 		connector_id_to_output(&data->master.display,
 				       data->connector_id);
@@ -345,15 +505,25 @@ static void setcrtc_implicit_plane(data_t *data)
 	do_or_die(create_lease(data->master.fd, &mcl));
 	drmSetClientCap(data->master.fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);
 
+	/*
+	 * For Legacy commit, If userspace wants to send modes with aspect-ratio bits
+	 * then the client cap for aspect-ratio bits must be set.
+	 */
+	if (mode->flags & DRM_MODE_FLAG_PIC_AR_MASK) {
+		drmSetClientCap(mcl.fd, DRM_CLIENT_CAP_ASPECT_RATIO, 1);
+	}
+
 	/* Set a mode on the leased output */
 	igt_assert_eq(0, prepare_crtc(&data->master, data));
 
 	/* sanity check */
-	do_or_die(drmModeSetCrtc(data->master.fd, data->crtc_id, -1,
-				 0, 0, object_ids, 1, mode));
-	do_or_die(drmModeSetCrtc(mcl.fd, data->crtc_id, -1,
-				 0, 0, object_ids, 1, mode));
+	ret = drmModeSetCrtc(data->master.fd, data->crtc_id, -1,
+			     0, 0, object_ids, 1, mode);
+	ret_mcl = drmModeSetCrtc(mcl.fd, data->crtc_id, -1,
+				 0, 0, object_ids, 1, mode);
 	close(mcl.fd);
+	igt_assert_eq(ret, 0);
+	igt_assert_eq(ret_mcl, 0);
 
 	object_ids[mcl.object_count++] = wrong_plane_id;
 	do_or_die(create_lease(data->master.fd, &mcl));
@@ -1071,7 +1241,7 @@ static void multimaster_lease(data_t *data)
 	drmSetClientCap(master2_fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);
 	lease2_fd = create_simple_lease(master2_fd, data);
 
-	close(master2_fd); /* close is an implicit DropMaster */
+	drm_close_driver(master2_fd); /* close is an implicit DropMaster */
 	igt_assert(!is_master(lease2_fd));
 
 	igt_device_set_master(data->master.fd);
@@ -1213,10 +1383,14 @@ igt_main
 			igt_describe(f->desc);
 			igt_subtest_with_dynamic_f("%s", f->name) {
 				for_each_pipe_with_valid_output(display, data.pipe, output) {
+					igt_display_reset(display);
+
+					igt_output_set_pipe(output, data.pipe);
+					if (!i915_pipe_output_combo_valid(display))
+						continue;
+
 					igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(data.pipe),
 						      igt_output_name(output)) {
-						igt_display_reset(display);
-						igt_display_commit(display);
 						data.crtc_id = display->pipes[data.pipe].crtc_id;
 						data.connector_id = output->id;
 						data.plane_id =
@@ -1263,6 +1437,6 @@ igt_main
 
 	igt_fixture {
 		igt_display_fini(display);
-		close(data.master.fd);
+		drm_close_driver(data.master.fd);
 	}
 }
