@@ -22,6 +22,12 @@
  *
  */
 
+/**
+ * TEST: kms plane lowres
+ * Category: Display
+ * Description: Test atomic mode setting with a plane by switching between high
+ *              and low resolutions
+ */
 #include "igt.h"
 #include "drmtest.h"
 #include <errno.h>
@@ -52,6 +58,7 @@ typedef struct {
 		igt_crc_t crc;
 	} ref_hires;
 	int x, y;
+	igt_pipe_crc_t *pipe_crc;
 } data_t;
 
 static drmModeModeInfo
@@ -59,40 +66,36 @@ get_lowres_mode(int drmfd, igt_output_t *output,
 		const drmModeModeInfo *mode_default)
 {
 	const drmModeModeInfo *mode;
-	bool found = false;
-	int limit = mode_default->vdisplay - SIZE;
+	const drmModeModeInfo *min;
 	int j;
 
+        /* search for lowest mode */
+        min = mode_default;
 	for (j = 0; j < output->config.connector->count_modes; j++) {
 		mode = &output->config.connector->modes[j];
-		if (mode->vdisplay < limit) {
-			found = true;
-			break;
-		}
+		if (mode->vdisplay < min->vdisplay)
+			min = mode;
 	}
 
-	if (!found) {
-		igt_require_f(mode_default->vdisplay - SIZE > 768,
-			      "Current mode not tall enough; plane would still be onscreen after switch to 10x7.\n");
+	igt_require_f(mode_default->vdisplay - min->vdisplay > 2 * SIZE,
+		      "Current mode for output %s not tall enough; "
+		      "plane would still be onscreen after switching to lowest mode.\n", output->name);
 
-		return *igt_std_1024_mode_get();
-	}
-
-	return *mode;
+	return *min;
 }
 
 static igt_plane_t *first_sdr_plane(igt_output_t *output, uint32_t devid)
 {
         int index;
 
-        index = intel_gen(devid) <= 9 ? 0 : SDR_PLANE_BASE;
+        index = intel_display_ver(devid) <= 9 ? 0 : SDR_PLANE_BASE;
 
         return igt_output_get_plane(output, index);
 }
 
 static bool is_sdr_plane(const igt_plane_t *plane, uint32_t devid)
 {
-        if (intel_gen(devid) <= 9)
+        if (intel_display_ver(devid) <= 9)
                 return true;
 
         return plane->index >= SDR_PLANE_BASE;
@@ -166,13 +169,10 @@ test_planes_on_pipe_with_output(data_t *data, igt_plane_t *plane, uint64_t modif
 {
 	const drmModeModeInfo *mode;
 	drmModeModeInfo mode_lowres;
-	igt_pipe_crc_t *pipe_crc;
 	unsigned tested = 0;
 	igt_plane_t *primary;
 	igt_crc_t crc_lowres, crc_hires1, crc_hires2;
 
-	igt_output_set_pipe(data->output, data->pipe);
-
 	primary = compatible_main_plane(plane, data->output, data->devid);
 	mode = igt_output_get_mode(data->output);
 	mode_lowres = get_lowres_mode(data->drm_fd, data->output, mode);
@@ -196,18 +196,15 @@ test_planes_on_pipe_with_output(data_t *data, igt_plane_t *plane, uint64_t modif
 	create_ref_fb(data, modifier, mode, &data->ref_hires.fb);
 	create_ref_fb(data, modifier, &mode_lowres, &data->ref_lowres.fb);
 
-	pipe_crc = igt_pipe_crc_new(data->drm_fd, data->pipe,
-				    INTEL_PIPE_CRC_SOURCE_AUTO);
-
 	igt_output_override_mode(data->output, &mode_lowres);
 	igt_plane_set_fb(primary, &data->ref_lowres.fb);
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
-	igt_pipe_crc_collect_crc(pipe_crc, &data->ref_lowres.crc);
+	igt_pipe_crc_collect_crc(data->pipe_crc, &data->ref_lowres.crc);
 
 	igt_output_override_mode(data->output, NULL);
 	igt_plane_set_fb(primary, &data->ref_hires.fb);
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
-	igt_pipe_crc_collect_crc(pipe_crc, &data->ref_hires.crc);
+	igt_pipe_crc_collect_crc(data->pipe_crc, &data->ref_hires.crc);
 
 	igt_plane_set_fb(primary, &data->fb_primary);
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
@@ -218,19 +215,19 @@ test_planes_on_pipe_with_output(data_t *data, igt_plane_t *plane, uint64_t modif
 
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
 
-	igt_pipe_crc_collect_crc(pipe_crc, &crc_hires1);
+	igt_pipe_crc_collect_crc(data->pipe_crc, &crc_hires1);
 
 	/* switch to lower resolution */
 	igt_output_override_mode(data->output, &mode_lowres);
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
 
-	igt_pipe_crc_collect_crc(pipe_crc, &crc_lowres);
+	igt_pipe_crc_collect_crc(data->pipe_crc, &crc_lowres);
 
 	/* switch back to higher resolution */
 	igt_output_override_mode(data->output, NULL);
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
 
-	igt_pipe_crc_collect_crc(pipe_crc, &crc_hires2);
+	igt_pipe_crc_collect_crc(data->pipe_crc, &crc_hires2);
 
 	igt_assert_crc_equal(&data->ref_hires.crc, &crc_hires1);
 	igt_assert_crc_equal(&data->ref_hires.crc, &crc_hires2);
@@ -239,10 +236,7 @@ test_planes_on_pipe_with_output(data_t *data, igt_plane_t *plane, uint64_t modif
 	igt_plane_set_fb(plane, NULL);
 	tested++;
 
-	igt_pipe_crc_free(pipe_crc);
-
 	igt_plane_set_fb(primary, NULL);
-	igt_output_set_pipe(data->output, PIPE_NONE);
 
 	igt_remove_fb(data->drm_fd, &data->fb_plane[1]);
 	igt_remove_fb(data->drm_fd, &data->fb_plane[0]);
@@ -259,70 +253,122 @@ test_planes_on_pipe(data_t *data, uint64_t modifier)
 	igt_plane_t *plane;
 	unsigned tested = 0;
 
-	igt_require_pipe(&data->display, data->pipe);
-	igt_display_require_output_on_pipe(&data->display, data->pipe);
-	igt_skip_on(!igt_display_has_format_mod(&data->display,
-						DRM_FORMAT_XRGB8888, modifier));
-
-	data->output = igt_get_single_output_for_pipe(&data->display, data->pipe);
-	igt_require(data->output);
-
-	igt_info("Testing connector %s using pipe %s\n",
-		 igt_output_name(data->output), kmstest_pipe_name(data->pipe));
-
-	for_each_plane_on_pipe(&data->display, data->pipe, plane) {
-		data->output = igt_get_single_output_for_pipe(&data->display, data->pipe);
-		igt_require(data->output);
-
+	for_each_plane_on_pipe(&data->display, data->pipe, plane)
 		tested += test_planes_on_pipe_with_output(data, plane, modifier);
-	}
 
 	igt_assert(tested > 0);
+}
+
+static void test_cleanup(data_t *data)
+{
+	igt_pipe_crc_free(data->pipe_crc);
+
 	igt_output_set_pipe(data->output, PIPE_NONE);
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
 }
 
+/**
+ * SUBTEST: tiling-none
+ * Description: Tests the visibility of the planes when switching between high
+ *              and low resolution with Linear buffer (no tiling)
+ * Driver requirement: i915, xe
+ * Functionality: plane
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: tiling-%s
+ * Description: Tests the visibility of the planes when switching between high
+ *              and low resolution with %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: plane, tiling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @4:           4-tiling
+ * @x:           x-tiling
+ * @y:           y-tiling
+ * @yf:          yf-tiling
+ */
+static void run_test(data_t *data, uint64_t modifier)
+{
+	enum pipe pipe;
+	igt_output_t *output;
+
+	if(!igt_display_has_format_mod(&data->display, DRM_FORMAT_XRGB8888, modifier))
+		return;
+
+	for_each_pipe(&data->display, pipe) {
+		for_each_valid_output_on_pipe(&data->display, pipe, output) {
+			data->pipe = pipe;
+			data->output = output;
+
+			igt_display_reset(&data->display);
+			igt_output_set_pipe(data->output, data->pipe);
+
+			if (!i915_pipe_output_combo_valid(&data->display))
+				continue;
+
+			data->pipe_crc = igt_pipe_crc_new(data->drm_fd, data->pipe,
+							  IGT_PIPE_CRC_SOURCE_AUTO);
+
+			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), data->output->name)
+				test_planes_on_pipe(data, modifier);
+
+			test_cleanup(data);
+		}
+	}
+}
+
+static const struct {
+	const char *name;
+	uint64_t modifier;
+} subtests[] = {
+	{ .name = "tiling-none",
+	  .modifier = DRM_FORMAT_MOD_LINEAR,
+	},
+	{ .name = "tiling-x",
+	  .modifier = I915_FORMAT_MOD_X_TILED,
+	},
+	{ .name = "tiling-y",
+	  .modifier = I915_FORMAT_MOD_Y_TILED,
+	},
+	{ .name = "tiling-yf",
+	  .modifier = I915_FORMAT_MOD_Yf_TILED,
+	},
+	{ .name = "tiling-4",
+	  .modifier = I915_FORMAT_MOD_4_TILED,
+	},
+};
+
 igt_main
 {
 	data_t data = {};
-	enum pipe pipe;
 
 	igt_fixture {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
-		data.devid = is_i915_device(data.drm_fd) ?
+		data.devid = is_intel_device(data.drm_fd) ?
 			intel_get_drm_devid(data.drm_fd) : 0;
 
 		kmstest_set_vt_graphics_mode();
 
 		igt_require_pipe_crc(data.drm_fd);
 		igt_display_require(&data.display, data.drm_fd);
+		igt_display_require_output(&data.display);
 		igt_require(data.display.is_atomic);
 	}
 
-	for_each_pipe_static(pipe) {
-		data.pipe = pipe;
-		igt_describe("Tests the visibility of the planes when switching between "
-			     "high and low resolution with tiling as none.");
-		igt_subtest_f("pipe-%s-tiling-none", kmstest_pipe_name(pipe))
-			test_planes_on_pipe(&data, LOCAL_DRM_FORMAT_MOD_NONE);
+	for (int i = 0; i < ARRAY_SIZE(subtests); i++) {
+		igt_describe_f("Tests the visibility of the planes when switching between "
+			       "high and low resolution with %s\n", subtests[i].name);
 
-		igt_describe("Tests the visibility of the planes when switching between "
-			     "high and low resolution with x-tiling.");
-		igt_subtest_f("pipe-%s-tiling-x", kmstest_pipe_name(pipe))
-			test_planes_on_pipe(&data, LOCAL_I915_FORMAT_MOD_X_TILED);
-
-		igt_describe("Tests the visibility of the planes when switching between "
-			     "high and low resolution with y-tiling.");
-		igt_subtest_f("pipe-%s-tiling-y", kmstest_pipe_name(pipe))
-			test_planes_on_pipe(&data, LOCAL_I915_FORMAT_MOD_Y_TILED);
-
-		igt_describe("Tests the visibility of the planes when switching between "
-			     "high and low resolution with yf-tiling.");
-		igt_subtest_f("pipe-%s-tiling-yf", kmstest_pipe_name(pipe))
-			test_planes_on_pipe(&data, LOCAL_I915_FORMAT_MOD_Yf_TILED);
+		igt_subtest_with_dynamic(subtests[i].name)
+			run_test(&data, subtests[i].modifier);
 	}
 
 	igt_fixture {
 		igt_display_fini(&data.display);
+		drm_close_driver(data.drm_fd);
 	}
 }
