@@ -22,6 +22,11 @@
  *
  */
 
+/**
+ * TEST: kms plane scaling
+ * Category: Display
+ * Description: Test display plane scaling
+ */
 #include "igt.h"
 #include "igt_vec.h"
 #include <math.h>
@@ -31,7 +36,8 @@ IGT_TEST_DESCRIPTION("Test display plane scaling");
 enum scaler_combo_test_type {
 	TEST_PLANES_UPSCALE = 0,
 	TEST_PLANES_DOWNSCALE,
-	TEST_PLANES_UPSCALE_DOWNSCALE
+	TEST_PLANES_UPSCALE_DOWNSCALE,
+	TEST_PLANES_DOWNSCALE_UPSCALE,
 };
 
 typedef struct {
@@ -42,6 +48,22 @@ typedef struct {
 	bool extended;
 } data_t;
 
+struct invalid_paramtests {
+	const char *testname;
+	uint32_t planesize[2];
+	struct {
+		enum igt_atomic_plane_properties prop;
+		uint32_t value;
+	} params[8];
+};
+
+static const struct invalid_paramtests intel_paramtests[] = {
+	{
+		.testname = "intel-max-src-size",
+		.planesize = {3840, 2160},
+	},
+};
+
 const struct {
 	const char * const describe;
 	const char * const name;
@@ -113,7 +135,7 @@ const struct {
 	{
 		"Tests downscaling with rotation for 0.5 scaling factor.",
 		"plane-downscale-with-rotation-factor-0-5",
-		0.25,
+		0.5,
 		false,
 	},
 	{
@@ -286,6 +308,69 @@ const struct {
 		0.75,
 		TEST_PLANES_UPSCALE_DOWNSCALE,
 	},
+	{
+		"Tests downscaling (scaling factor 0.25) and upscaling (20x20) of 2 planes.",
+		"planes-downscale-factor-0-25-upscale-20x20",
+		0.25,
+		0.0,
+		TEST_PLANES_DOWNSCALE_UPSCALE,
+	},
+	{
+		"Tests downscaling (scaling factor 0.25) and upscaling (scaling factor 0.25) of 2 planes.",
+		"planes-downscale-factor-0-25-upscale-factor-0-25",
+		0.25,
+		0.25,
+		TEST_PLANES_DOWNSCALE_UPSCALE,
+	},
+	{
+		"Tests downscaling (scaling factor 0.25) and scaling (unity) of 2 planes.",
+		"planes-downscale-factor-0-25-unity-scaling",
+		0.25,
+		1.0,
+		TEST_PLANES_DOWNSCALE_UPSCALE,
+	},
+	{
+		"Tests downscaling (scaling factor 0.5) and upscaling (20x20) of 2 planes.",
+		"planes-downscale-factor-0-5-upscale-20x20",
+		0.5,
+		0.0,
+		TEST_PLANES_DOWNSCALE_UPSCALE,
+	},
+	{
+		"Tests downscaling (scaling factor 0.5) and upscaling (scaling factor 0.25) of 2 planes.",
+		"planes-downscale-factor-0-5-upscale-factor-0-25",
+		0.5,
+		0.25,
+		TEST_PLANES_DOWNSCALE_UPSCALE,
+	},
+	{
+		"Tests downscaling (scaling factor 0.5) and scaling (unity) of 2 planes.",
+		"planes-downscale-factor-0-5-unity-scaling",
+		0.5,
+		1.0,
+		TEST_PLANES_DOWNSCALE_UPSCALE,
+	},
+	{
+		"Tests downscaling (scaling factor 0.75) and upscaling (20x20) of 2 planes.",
+		"planes-downscale-factor-0-75-upscale-20x20",
+		0.75,
+		0.0,
+		TEST_PLANES_DOWNSCALE_UPSCALE,
+	},
+	{
+		"Tests downscaling (scaling factor 0.75) and upscaling (scaling factor 0.25) of 2 planes.",
+		"planes-downscale-factor-0-75-upscale-factor-0-25",
+		0.75,
+		0.25,
+		TEST_PLANES_DOWNSCALE_UPSCALE,
+	},
+	{
+		"Tests downscaling (scaling factor 0.75) and scaling (unity) of 2 planes.",
+		"planes-downscale-factor-0-75-unity-scaling",
+		0.75,
+		1.0,
+		TEST_PLANES_DOWNSCALE_UPSCALE,
+	},
 };
 
 static int get_width(drmModeModeInfo *mode, double scaling_factor)
@@ -361,7 +446,8 @@ static void check_scaling_pipe_plane_rot(data_t *d, igt_plane_t *plane,
 	else
 		igt_plane_set_size(plane, width, height);
 
-	igt_plane_set_rotation(plane, rot);
+	if (rot != IGT_ROTATION_0)
+		igt_plane_set_rotation(plane, rot);
 	commit_ret = igt_display_try_commit2(display, COMMIT_ATOMIC);
 
 	igt_plane_set_fb(plane, NULL);
@@ -383,7 +469,7 @@ static const igt_rotation_t rotations[] = {
 
 static bool can_scale(data_t *d, unsigned format)
 {
-	if (!is_i915_device(d->drm_fd))
+	if (!is_intel_device(d->drm_fd))
 		return true;
 
 	switch (format) {
@@ -408,7 +494,7 @@ static bool test_format(data_t *data,
 	if (!igt_fb_supported_format(format))
 		return false;
 
-	if (!is_i915_device(data->drm_fd) ||
+	if (!is_intel_device(data->drm_fd) ||
 	    data->extended)
 		return true;
 
@@ -425,7 +511,7 @@ static bool test_format(data_t *data,
 
 static bool test_pipe_iteration(data_t *data, enum pipe pipe, int iteration)
 {
-	if (!is_i915_device(data->drm_fd) ||
+	if (!is_intel_device(data->drm_fd) ||
 	    data->extended)
 		return true;
 
@@ -443,6 +529,48 @@ static const uint64_t modifiers[] = {
 	I915_FORMAT_MOD_4_TILED
 };
 
+/**
+ * SUBTEST: plane-scaler-with-modifiers-unity-scaling
+ * Description: Tests scaling with modifiers, unity scaling.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: plane-scaler-with-clipping-clamping-modifiers
+ * Description: Tests scaling with clipping and clamping, modifiers.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: plane-upscale-with-modifiers-%s
+ * Description: Tests upscaling with modifiers %arg[1].
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @20x20:          from 20x20 fb
+ * @factor-0-25:    for 0.25 scaling factor
+ */
+
+/**
+ * SUBTEST: plane-downscale-with-modifiers-factor-%s
+ * Description: Tests downscaling with modifiers for %arg[1] scaling factor.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @0-25:   0.25
+ * @0-5:    0.5
+ * @0-75:   0.75
+ */
 static void test_scaler_with_modifier_pipe(data_t *d,
 					   int width, int height,
 					   bool is_upscale,
@@ -475,6 +603,48 @@ static void test_scaler_with_modifier_pipe(data_t *d,
 	}
 }
 
+/**
+ * SUBTEST: plane-scaler-with-rotation-unity-scaling
+ * Description: Tests scaling with rotation, unity scaling.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: plane-scaler-with-clipping-clamping-rotation
+ * Description: Tests scaling with clipping and clamping, rotation.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: plane-upscale-with-rotation-%s
+ * Description: Tests upscaling with rotation %arg[1].
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @20x20:          from 20x20 fb
+ * @factor-0-25:    for 0.25 scaling factor
+ */
+
+/**
+ * SUBTEST: plane-downscale-with-rotation-factor-%s
+ * Description: Tests downscaling with rotation for %arg[1] scaling factor.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @0-25:   0.25
+ * @0-5:    0.5
+ * @0-75:   0.75
+ */
 static void test_scaler_with_rotation_pipe(data_t *d,
 					   int width, int height,
 					   bool is_upscale,
@@ -508,6 +678,48 @@ static void test_scaler_with_rotation_pipe(data_t *d,
 	}
 }
 
+/**
+ * SUBTEST: plane-scaler-with-pixel-format-unity-scaling
+ * Description: Tests scaling with pixel formats, unity scaling.
+ * Driver requirement: i915, xe
+ * Functionality: pixel_formats, plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: plane-scaler-with-clipping-clamping-pixel-formats
+ * Description: Tests scaling with clipping and clamping, pixel formats.
+ * Driver requirement: i915, xe
+ * Functionality: pixel_formats, plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: plane-upscale-with-pixel-format-%s
+ * Description: Tests upscaling with pixel formats %arg[1].
+ * Driver requirement: i915, xe
+ * Functionality: pixel_formats, plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @20x20:          from 20x20 fb
+ * @factor-0-25:    for 0.25 scaling factor
+ */
+
+/**
+ * SUBTEST: plane-downscale-with-pixel-format-factor-%s
+ * Description: Tests downscaling with pixel formats for %arg[1] scaling factor.
+ * Driver requirement: i915, xe
+ * Functionality: pixel_formats, plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @0-25:   0.25
+ * @0-5:    0.5
+ * @0-75:   0.75
+ */
 static void test_scaler_with_pixel_format_pipe(data_t *d, int width, int height, bool is_upscale,
 					       enum pipe pipe, igt_output_t *output)
 {
@@ -547,30 +759,45 @@ static void test_scaler_with_pixel_format_pipe(data_t *d, int width, int height,
 	}
 }
 
-static void find_connected_pipe(igt_display_t *display, bool second, enum pipe *pipe, igt_output_t **output)
+static enum pipe
+find_connected_pipe(igt_display_t *display, bool second, igt_output_t **output)
 {
-	enum pipe first = PIPE_NONE;
-	igt_output_t *first_output = NULL;
+	enum pipe pipe;
+	bool first_output = false;
 	bool found = false;
 
-	for_each_pipe_with_valid_output(display, *pipe, *output) {
-		if (first == *pipe || *output == first_output)
-			continue;
+	igt_display_reset(display);
 
-		if (second) {
-			first = *pipe;
-			first_output = *output;
-			second = false;
-			continue;
+	for_each_pipe(display, pipe) {
+		for_each_valid_output_on_pipe(display, pipe, *output) {
+			if((*output)->pending_pipe != PIPE_NONE)
+				continue;
+
+			igt_output_set_pipe(*output, pipe);
+			if (i915_pipe_output_combo_valid(display)) {
+				found = true;
+
+				if (second) {
+					first_output = true;
+					second = false;
+					found = false;
+				}
+				break;
+			}
+			igt_output_set_pipe(*output, PIPE_NONE);
 		}
-
-		return;
+		if (found)
+			break;
 	}
 
+	igt_display_reset(display);
+
 	if (first_output)
 		igt_require_f(found, "No second valid output found\n");
 	else
 		igt_require_f(found, "No valid outputs found\n");
+
+	return pipe;
 }
 
 static void
@@ -589,23 +816,25 @@ __test_planes_scaling_combo(data_t *d, int w1, int h1, int w2, int h2,
 	igt_plane_set_fb(p1, fb1);
 	igt_plane_set_fb(p2, fb2);
 
-	if (test_type == TEST_PLANES_UPSCALE) {
-		/* first plane upscaling */
+	switch (test_type) {
+	case TEST_PLANES_UPSCALE:
 		igt_plane_set_size(p1, mode->hdisplay, mode->vdisplay);
-		/* second plane upscaling */
 		igt_plane_set_size(p2, mode->hdisplay - 20, mode->vdisplay - 20);
-	}
-	if (test_type == TEST_PLANES_DOWNSCALE) {
-		/* first plane downscaling */
+		break;
+	case TEST_PLANES_DOWNSCALE:
 		igt_plane_set_size(p1, w1, h1);
-		/* second plane downscaling */
 		igt_plane_set_size(p2, w2, h2);
-	}
-	if (test_type == TEST_PLANES_UPSCALE_DOWNSCALE) {
-		/* first plane upscaling */
+		break;
+	case TEST_PLANES_UPSCALE_DOWNSCALE:
 		igt_plane_set_size(p1, mode->hdisplay, mode->vdisplay);
-		/* second plane downscaling */
 		igt_plane_set_size(p2, w2, h2);
+		break;
+	case TEST_PLANES_DOWNSCALE_UPSCALE:
+		igt_plane_set_size(p1, w1, h1);
+		igt_plane_set_size(p2, mode->hdisplay, mode->vdisplay);
+		break;
+	default:
+		igt_assert(0);
 	}
 
 	ret = igt_display_try_commit_atomic(display, DRM_MODE_ATOMIC_ALLOW_MODESET, NULL);
@@ -618,16 +847,91 @@ __test_planes_scaling_combo(data_t *d, int w1, int h1, int w2, int h2,
 	igt_assert_eq(ret, 0);
 }
 
-static void setup_fb(int fd, int width, int height,
-		     double r, double g, double b,
-		     struct igt_fb *fb)
+static void setup_fb(int fd, int width, int height, struct igt_fb *fb)
 {
-	igt_create_color_pattern_fb(fd, width, height,
-				    DRM_FORMAT_XRGB8888,
-				    I915_TILING_NONE,
-				    r, g, b, fb);
+	igt_create_fb(fd, width, height,
+		      DRM_FORMAT_XRGB8888,
+		      DRM_FORMAT_MOD_LINEAR,
+		      fb);
 }
 
+/**
+ * SUBTEST: planes-downscale-factor-%s
+ * Description: Tests downscaling of 2 planes for %arg[1] scaling factor.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @0-25:  0.25
+ * @0-5:   0.5
+ * @0-75:  0.75
+ */
+/**
+ * SUBTEST: planes-downscale-factor-%s-%s
+ * Description: Tests downscaling (scaling factor %arg[1]) and upscaling (%arg[2])
+ *              of 2 planes.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @0-25:  0.25
+ * @0-5:   0.5
+ * @0-75:  0.75
+ *
+ * arg[2]:
+ *
+ * @upscale-20x20:           upscale 20x20
+ * @upscale-factor-0-25:     scaling factor 0.25
+ * @unity-scaling:           Unity
+ */
+/**
+ * SUBTEST: planes-scaler-unity-scaling
+ * Description: Tests scaling of 2 planes, unity scaling.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: planes-upscale-%s
+ * Description: Tests upscaling of 2 planes %arg[1].
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @20x20:          from 20x20 fb
+ * @factor-0-25:    for 0.25 scaling factor
+ */
+/**
+ * SUBTEST: planes-%s-downscale-factor-%s
+ * Description: Tests scaling (%arg[1]) and downscaling (scaling factor %arg[2])
+ *              of 2 planes.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @unity-scaling:           Unity
+ * @upscale-factor-0-25:     scaling factor 0.25
+ * @upscale-20x20:           upscale 20x20
+ *
+ * arg[2]:
+ *
+ * @0-25:  0.25
+ * @0-5:   0.5
+ * @0-75:  0.75
+ */
+
 static void
 test_planes_scaling_combo(data_t *d, int w1, int h1, int w2, int h2,
 			  enum pipe pipe, igt_output_t *output,
@@ -635,26 +939,38 @@ test_planes_scaling_combo(data_t *d, int w1, int h1, int w2, int h2,
 {
 	igt_display_t *display = &d->display;
 	drmModeModeInfo *mode;
+	int n_planes;
 
 	cleanup_crtc(d);
 
 	igt_output_set_pipe(output, pipe);
 	mode = igt_output_get_mode(output);
 
-	if (test_type == TEST_PLANES_UPSCALE) {
-		setup_fb(display->drm_fd, w1, h1, 1.0, 0.0, 0.0, &d->fb[1]);
-		setup_fb(display->drm_fd, w2, h2, 0.0, 1.0, 0.0, &d->fb[2]);
-	}
-	if (test_type == TEST_PLANES_DOWNSCALE) {
-		setup_fb(display->drm_fd, mode->hdisplay, mode->vdisplay, 1.0, 0.0, 0.0, &d->fb[1]);
-		setup_fb(display->drm_fd, mode->hdisplay, mode->vdisplay, 0.0, 1.0, 0.0, &d->fb[2]);
-	}
-	if (test_type == TEST_PLANES_UPSCALE_DOWNSCALE) {
-		setup_fb(display->drm_fd, w1, h1, 1.0, 0.0, 0.0, &d->fb[1]);
-		setup_fb(display->drm_fd, mode->hdisplay, mode->vdisplay, 0.0, 1.0, 0.0, &d->fb[2]);
+	n_planes = display->pipes[pipe].n_planes;
+	igt_require(n_planes >= 2);
+
+	switch (test_type) {
+	case TEST_PLANES_UPSCALE:
+		setup_fb(display->drm_fd, w1, h1, &d->fb[1]);
+		setup_fb(display->drm_fd, w2, h2, &d->fb[2]);
+		break;
+	case TEST_PLANES_DOWNSCALE:
+		setup_fb(display->drm_fd, mode->hdisplay, mode->vdisplay, &d->fb[1]);
+		setup_fb(display->drm_fd, mode->hdisplay, mode->vdisplay, &d->fb[2]);
+		break;
+	case TEST_PLANES_UPSCALE_DOWNSCALE:
+		setup_fb(display->drm_fd, w1, h1, &d->fb[1]);
+		setup_fb(display->drm_fd, mode->hdisplay, mode->vdisplay, &d->fb[2]);
+		break;
+	case TEST_PLANES_DOWNSCALE_UPSCALE:
+		setup_fb(display->drm_fd, mode->hdisplay, mode->vdisplay, &d->fb[1]);
+		setup_fb(display->drm_fd, w2, h2, &d->fb[2]);
+		break;
+	default:
+		igt_assert(0);
 	}
 
-	for (int k = 0; k < display->pipes[pipe].n_planes; k++) {
+	for (int k = 0; k < n_planes - 1; k += 2) {
 		igt_plane_t *p1, *p2;
 
 		p1 = &display->pipes[pipe].planes[k];
@@ -674,6 +990,14 @@ test_planes_scaling_combo(data_t *d, int w1, int h1, int w2, int h2,
 	cleanup_fbs(d);
 }
 
+/**
+ * SUBTEST: invalid-num-scalers
+ * Description: Negative test for number of scalers per pipe.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void
 test_invalid_num_scalers(data_t *d, enum pipe pipe, igt_output_t *output)
 {
@@ -698,21 +1022,21 @@ test_invalid_num_scalers(data_t *d, enum pipe pipe, igt_output_t *output)
 	plane[2] = igt_pipe_get_plane_type_index(pipe_obj, DRM_PLANE_TYPE_OVERLAY, 2);
 	igt_require(plane[2]);
 
-	igt_create_color_pattern_fb(display->drm_fd,
-                                    width, height,
-                                    DRM_FORMAT_XRGB8888,
-                                    I915_TILING_NONE,
-                                    1.0, 0.0, 0.0, &d->fb[0]);
-	igt_create_color_pattern_fb(display->drm_fd,
-                                    width, height,
-                                    DRM_FORMAT_XRGB8888,
-                                    I915_TILING_NONE,
-                                    0.0, 1.0, 0.0, &d->fb[1]);
-	igt_create_color_pattern_fb(display->drm_fd,
-                                    width, height,
-                                    DRM_FORMAT_XRGB8888,
-                                    I915_TILING_NONE,
-                                    0.0, 0.0, 1.0, &d->fb[2]);
+	igt_create_fb(display->drm_fd,
+		      width, height,
+		      DRM_FORMAT_XRGB8888,
+		      DRM_FORMAT_MOD_LINEAR,
+		      &d->fb[0]);
+	igt_create_fb(display->drm_fd,
+		      width, height,
+		      DRM_FORMAT_XRGB8888,
+		      DRM_FORMAT_MOD_LINEAR,
+		      &d->fb[1]);
+	igt_create_fb(display->drm_fd,
+		      width, height,
+		      DRM_FORMAT_XRGB8888,
+		      DRM_FORMAT_MOD_LINEAR,
+		      &d->fb[2]);
 
 	igt_plane_set_fb(plane[0], &d->fb[0]);
 	igt_plane_set_fb(plane[1], &d->fb[1]);
@@ -722,14 +1046,15 @@ test_invalid_num_scalers(data_t *d, enum pipe pipe, igt_output_t *output)
 	igt_plane_set_size(plane[1], mode->hdisplay, mode->vdisplay);
 	igt_plane_set_size(plane[2], mode->hdisplay, mode->vdisplay);
 
-	/* This commit is expected to fail for i915 devices. i915 devices support
+	/*
+	 * This commit is expected to fail for intel devices. intel devices support
 	 * max 2 scalers/pipe. In dmesg we can find: Too many scaling requests 3 > 2.
-	 * For devices (non-i915, or possible future i915) that are able to perform this
-	 * amount of scaling; handle that case aswell.
-	 * */
+	 * For devices (non-intel, or possible future intel) that are able to perform
+	 * this amount of scaling; handle that case aswell.
+	 */
 	ret = igt_display_try_commit_atomic(display, DRM_MODE_ATOMIC_ALLOW_MODESET, NULL);
 	igt_skip_on_f(ret == 0, "Cannot test handling of too many scaling ops, the device supports a large amount.\n");
-	igt_assert_eq(ret, -EINVAL);
+	igt_assert(ret == -EINVAL || ret == -ERANGE);
 
 	/* cleanup */
 	igt_plane_set_fb(plane[0], NULL);
@@ -738,6 +1063,14 @@ test_invalid_num_scalers(data_t *d, enum pipe pipe, igt_output_t *output)
 	cleanup_fbs(d);
 }
 
+/**
+ * SUBTEST: 2x-scaler-multi-pipe
+ * Description: Tests scaling with multi-pipe.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 static void test_scaler_with_multi_pipe_plane(data_t *d)
 {
 	igt_display_t *display = &d->display;
@@ -747,13 +1080,16 @@ static void test_scaler_with_multi_pipe_plane(data_t *d)
 	enum pipe pipe1, pipe2;
 	int ret1, ret2;
 
-	cleanup_crtc(d);
-
-	find_connected_pipe(display, false, &pipe1, &output1);
-	find_connected_pipe(display, true, &pipe2, &output2);
+	cleanup_fbs(d);
 
+	pipe1 = find_connected_pipe(display, false, &output1);
+	pipe2 = find_connected_pipe(display, true, &output2);
 	igt_skip_on(!output1 || !output2);
 
+	igt_info("Using (pipe %s + %s) and (pipe %s + %s) to run the subtest.\n",
+		 kmstest_pipe_name(pipe1), igt_output_name(output1),
+		 kmstest_pipe_name(pipe2), igt_output_name(output2));
+
 	igt_output_set_pipe(output1, pipe1);
 	igt_output_set_pipe(output2, pipe2);
 
@@ -768,16 +1104,16 @@ static void test_scaler_with_multi_pipe_plane(data_t *d)
 
 	igt_create_pattern_fb(d->drm_fd, 600, 600,
 			      DRM_FORMAT_XRGB8888,
-			      I915_TILING_NONE, &d->fb[0]);
+			      DRM_FORMAT_MOD_LINEAR, &d->fb[0]);
 	igt_create_pattern_fb(d->drm_fd, 500, 500,
 			      DRM_FORMAT_XRGB8888,
-			      I915_TILING_NONE, &d->fb[1]);
+			      DRM_FORMAT_MOD_LINEAR, &d->fb[1]);
 	igt_create_pattern_fb(d->drm_fd, 700, 700,
 			      DRM_FORMAT_XRGB8888,
-			      I915_TILING_NONE, &d->fb[2]);
+			      DRM_FORMAT_MOD_LINEAR, &d->fb[2]);
 	igt_create_pattern_fb(d->drm_fd, 400, 400,
 			      DRM_FORMAT_XRGB8888,
-			      I915_TILING_NONE, &d->fb[3]);
+			      DRM_FORMAT_MOD_LINEAR, &d->fb[3]);
 
 	igt_plane_set_fb(plane[0], &d->fb[0]);
 	igt_plane_set_fb(plane[1], &d->fb[1]);
@@ -814,11 +1150,186 @@ static void test_scaler_with_multi_pipe_plane(data_t *d)
 	cleanup_fbs(d);
 
 	igt_skip_on_f(ret1 == -ERANGE || ret1 == -EINVAL ||
-		      ret2 == -ERANGE || ret1 == -EINVAL,
+		      ret2 == -ERANGE || ret2 == -EINVAL,
 		      "Scaling op is not supported by driver\n");
 	igt_assert_eq(ret1 && ret2, 0);
 }
 
+/**
+ * SUBTEST: invalid-parameters
+ * Description: Test parameters which should not be accepted
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+static void invalid_parameter_tests(data_t *d)
+{
+	enum pipe pipe = PIPE_A;
+	igt_output_t *output;
+	igt_fb_t fb;
+	igt_plane_t *plane;
+	int rval;
+
+	static const struct invalid_paramtests paramtests[] = {
+		{
+			.testname = "less-than-1-height-src",
+			.planesize = {256, 8},
+			.params = {{IGT_PLANE_SRC_H, IGT_FIXED(0, 30) }, {~0}}
+		},
+		{
+			.testname = "less-than-1-width-src",
+			.planesize = {8, 256},
+			.params = {{IGT_PLANE_SRC_W, IGT_FIXED(0, 30) }, {~0}}
+		},
+	};
+
+	igt_fixture {
+		output = igt_get_single_output_for_pipe(&d->display, pipe);
+		igt_require(output);
+
+		igt_output_set_pipe(output, pipe);
+		plane = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+
+		igt_create_fb(d->drm_fd, 256, 256,
+			      DRM_FORMAT_XRGB8888,
+			      DRM_FORMAT_MOD_LINEAR,
+			      &fb);
+	}
+
+	igt_describe("test parameters which should not be accepted");
+	igt_subtest_with_dynamic("invalid-parameters") {
+		for (uint32_t i = 0; i < ARRAY_SIZE(paramtests); i++) {
+			igt_dynamic(paramtests[i].testname) {
+				igt_plane_set_position(plane, 0, 0);
+				igt_plane_set_fb(plane, &fb);
+				igt_plane_set_size(plane,
+						   paramtests[i].planesize[0],
+						   paramtests[i].planesize[1]);
+
+				for (uint32_t j = 0; paramtests[i].params[j].prop != ~0; j++)
+					igt_plane_set_prop_value(plane,
+								 paramtests[i].params[j].prop,
+								 paramtests[i].params[j].value);
+
+				rval = igt_display_try_commit2(&d->display, COMMIT_ATOMIC);
+
+				igt_assert(rval == -EINVAL || rval == -ERANGE);
+			}
+		}
+	}
+
+	igt_fixture {
+		igt_remove_fb(d->drm_fd, &fb);
+		igt_output_set_pipe(output, PIPE_NONE);
+	}
+}
+
+static drmModeModeInfo *find_mode(data_t *data, igt_output_t *output, const uint32_t planesize[])
+{
+	drmModeModeInfo *mode = NULL;
+
+	for (int i = 0; i < output->config.connector->count_modes; i++) {
+		if (output->config.connector->modes[i].hdisplay == planesize[0] &&
+		    output->config.connector->modes[i].vdisplay == planesize[1] ) {
+			if (mode &&
+			    mode->vrefresh < output->config.connector->modes[i].vrefresh)
+				continue;
+
+			mode = &output->config.connector->modes[i];
+		}
+	}
+
+	return mode;
+}
+
+/**
+ * SUBTEST: intel-max-src-size
+ * Description: Test for validating max source size.
+ * Driver requirement: i915, xe
+ * Functionality: plane, scaling
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
+/*
+ *	Max source/destination width/height for intel driver.
+ *	These numbers are coming from
+ *	drivers/gpu/drm/i915/display/skl_scaler.c in kernel sources.
+ *
+ *	DISPLAY_VER < 11
+ *		max_src_w = 4096
+ *		max_src_h = 4096
+ *		max_dst_w = 4096
+ *		max_dst_h = 4096
+ *
+ *	DISPLAY_VER = 11
+ *		max_src_w = 5120
+ *		max_src_h = 4096
+ *		max_dst_w = 5120
+ *		max_dst_h = 4096
+ *
+ *	DISPLAY_VER = 12-13
+ *		max_src_w = 5120
+ *		max_src_h = 8192
+ *		max_dst_w = 8192
+ *		max_dst_h = 8192
+ *
+ *	DISPLAY_VER = 14
+ *		max_src_w = 4096
+ *		max_src_h = 8192
+ *		max_dst_w = 8192
+ *		max_dst_h = 8192
+ */
+static void intel_max_source_size_test(data_t *d, enum pipe pipe, igt_output_t *output,
+				       drmModeModeInfo *mode, const uint32_t planesize[])
+{
+	igt_fb_t fb;
+	igt_plane_t *plane;
+	int rval;
+
+	cleanup_crtc(d);
+
+	igt_output_set_pipe(output, pipe);
+	igt_output_override_mode(output, mode);
+	plane = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+
+	igt_create_fb(d->drm_fd, 5120, 4320,
+		      DRM_FORMAT_XRGB8888,
+		      DRM_FORMAT_MOD_LINEAR,
+		      &fb);
+
+	igt_plane_set_position(plane, 0, 0);
+	igt_plane_set_fb(plane, &fb);
+	igt_plane_set_size(plane, planesize[0], planesize[1]);
+
+	rval = igt_display_try_commit2(&d->display, COMMIT_ATOMIC);
+
+	if (intel_display_ver(d->devid) < 11 || intel_display_ver(d->devid) >= 14)
+		igt_assert_eq(rval, -EINVAL);
+	else
+		igt_assert_eq(rval, 0);
+
+	igt_plane_set_fb(plane, NULL);
+	cleanup_fbs(d);
+}
+
+static bool
+pipe_output_combo_valid(igt_display_t *display,
+			enum pipe pipe, igt_output_t *output)
+{
+	bool ret = true;
+
+	igt_display_reset(display);
+
+	igt_output_set_pipe(output, pipe);
+	if (!i915_pipe_output_combo_valid(display))
+		ret = false;
+	igt_output_set_pipe(output, PIPE_NONE);
+
+	return ret;
+}
+
 static int opt_handler(int opt, int opt_index, void *_data)
 {
 	data_t *data = _data;
@@ -849,7 +1360,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 	igt_fixture {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 		igt_display_require(&data.display, data.drm_fd);
-		data.devid = is_i915_device(data.drm_fd) ?
+		data.devid = is_intel_device(data.drm_fd) ?
 			intel_get_drm_devid(data.drm_fd) : 0;
 		igt_require(data.display.is_atomic);
 	}
@@ -860,17 +1371,22 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 		for (int index = 0; index < ARRAY_SIZE(scaler_with_pixel_format_tests); index++) {
 			igt_describe(scaler_with_pixel_format_tests[index].describe);
 			igt_subtest_with_dynamic(scaler_with_pixel_format_tests[index].name) {
-			for_each_pipe_with_single_output(&data.display, pipe, output)
-				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
-					drmModeModeInfo *mode;
+				for_each_pipe(&data.display, pipe) {
+					for_each_valid_output_on_pipe(&data.display, pipe, output) {
+						if (!pipe_output_combo_valid(&data.display, pipe, output))
+							continue;
 
-					mode = igt_output_get_mode(output);
+						igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
+							drmModeModeInfo *mode = igt_output_get_mode(output);
 
-					test_scaler_with_pixel_format_pipe(&data,
-							get_width(mode, scaler_with_pixel_format_tests[index].sf),
-							get_height(mode, scaler_with_pixel_format_tests[index].sf),
-							scaler_with_pixel_format_tests[index].is_upscale,
-							pipe, output);
+							test_scaler_with_pixel_format_pipe(&data,
+								get_width(mode, scaler_with_pixel_format_tests[index].sf),
+								get_height(mode, scaler_with_pixel_format_tests[index].sf),
+								scaler_with_pixel_format_tests[index].is_upscale,
+								pipe, output);
+						}
+						break;
+					}
 				}
 			}
 		}
@@ -878,17 +1394,22 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 		for (int index = 0; index < ARRAY_SIZE(scaler_with_rotation_tests); index++) {
 			igt_describe(scaler_with_rotation_tests[index].describe);
 			igt_subtest_with_dynamic(scaler_with_rotation_tests[index].name) {
-			for_each_pipe_with_single_output(&data.display, pipe, output)
-				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
-					drmModeModeInfo *mode;
+				for_each_pipe(&data.display, pipe) {
+					for_each_valid_output_on_pipe(&data.display, pipe, output) {
+						if (!pipe_output_combo_valid(&data.display, pipe, output))
+							continue;
 
-					mode = igt_output_get_mode(output);
+						igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
+							drmModeModeInfo *mode = igt_output_get_mode(output);
 
-					test_scaler_with_rotation_pipe(&data,
-							get_width(mode, scaler_with_rotation_tests[index].sf),
-							get_height(mode, scaler_with_rotation_tests[index].sf),
-							scaler_with_rotation_tests[index].is_upscale,
-							pipe, output);
+							test_scaler_with_rotation_pipe(&data,
+								get_width(mode, scaler_with_rotation_tests[index].sf),
+								get_height(mode, scaler_with_rotation_tests[index].sf),
+								scaler_with_rotation_tests[index].is_upscale,
+								pipe, output);
+						}
+						break;
+					}
 				}
 			}
 		}
@@ -896,94 +1417,150 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 		for (int index = 0; index < ARRAY_SIZE(scaler_with_modifiers_tests); index++) {
 			igt_describe(scaler_with_modifiers_tests[index].describe);
 			igt_subtest_with_dynamic(scaler_with_modifiers_tests[index].name) {
-			for_each_pipe_with_single_output(&data.display, pipe, output)
-				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
-					drmModeModeInfo *mode;
+				for_each_pipe(&data.display, pipe) {
+					for_each_valid_output_on_pipe(&data.display, pipe, output) {
+						if (!pipe_output_combo_valid(&data.display, pipe, output))
+							continue;
 
-					mode = igt_output_get_mode(output);
+						igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
+							drmModeModeInfo *mode = igt_output_get_mode(output);
 
-					test_scaler_with_modifier_pipe(&data,
-							get_width(mode, scaler_with_rotation_tests[index].sf),
-							get_height(mode, scaler_with_rotation_tests[index].sf),
-							scaler_with_rotation_tests[index].is_upscale,
-							pipe, output);
+							test_scaler_with_modifier_pipe(&data,
+								get_width(mode, scaler_with_modifiers_tests[index].sf),
+								get_height(mode, scaler_with_modifiers_tests[index].sf),
+								scaler_with_modifiers_tests[index].is_upscale,
+								pipe, output);
+						}
+						break;
+					}
 				}
 			}
 		}
 
 		igt_describe("Tests scaling with clipping and clamping, pixel formats.");
 		igt_subtest_with_dynamic("plane-scaler-with-clipping-clamping-pixel-formats") {
-			for_each_pipe_with_single_output(&data.display, pipe, output) {
-				drmModeModeInfo *mode;
+			for_each_pipe(&data.display, pipe) {
+				for_each_valid_output_on_pipe(&data.display, pipe, output) {
+					if (!pipe_output_combo_valid(&data.display, pipe, output))
+						continue;
 
-				mode = igt_output_get_mode(output);
+					igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
+						drmModeModeInfo *mode = igt_output_get_mode(output);
 
-				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
-					test_scaler_with_pixel_format_pipe(&data, mode->hdisplay + 100,
+						test_scaler_with_pixel_format_pipe(&data, mode->hdisplay + 100,
 							mode->vdisplay + 100, false, pipe, output);
+					}
+					break;
+				}
 			}
 		}
 
 		igt_describe("Tests scaling with clipping and clamping, rotation.");
 		igt_subtest_with_dynamic("plane-scaler-with-clipping-clamping-rotation") {
-			for_each_pipe_with_single_output(&data.display, pipe, output) {
-				drmModeModeInfo *mode;
+			for_each_pipe(&data.display, pipe) {
+				for_each_valid_output_on_pipe(&data.display, pipe, output) {
+					if (!pipe_output_combo_valid(&data.display, pipe, output))
+						continue;
 
-				mode = igt_output_get_mode(output);
+					igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
+						drmModeModeInfo *mode = igt_output_get_mode(output);
 
-				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
-					test_scaler_with_rotation_pipe(&data, mode->hdisplay + 100,
+						test_scaler_with_rotation_pipe(&data, mode->hdisplay + 100,
 							mode->vdisplay + 100, false, pipe, output);
+					}
+					break;
+				}
 			}
 		}
 
 		igt_describe("Tests scaling with clipping and clamping, modifiers.");
 		igt_subtest_with_dynamic("plane-scaler-with-clipping-clamping-modifiers") {
-			for_each_pipe_with_single_output(&data.display, pipe, output) {
-				drmModeModeInfo *mode;
+			for_each_pipe(&data.display, pipe) {
+				for_each_valid_output_on_pipe(&data.display, pipe, output) {
+					if (!pipe_output_combo_valid(&data.display, pipe, output))
+						continue;
 
-				mode = igt_output_get_mode(output);
-
-				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
-					test_scaler_with_modifier_pipe(&data, mode->hdisplay + 100,
+					igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
+						drmModeModeInfo *mode = igt_output_get_mode(output);
+						test_scaler_with_modifier_pipe(&data, mode->hdisplay + 100,
 							mode->vdisplay + 100, false, pipe, output);
+					}
+					break;
+				}
 			}
 		}
 
 		for (int index = 0; index < ARRAY_SIZE(scaler_with_2_planes_tests); index++) {
 			igt_describe(scaler_with_2_planes_tests[index].describe);
 			igt_subtest_with_dynamic(scaler_with_2_planes_tests[index].name) {
-			for_each_pipe_with_single_output(&data.display, pipe, output)
-				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
-					drmModeModeInfo *mode;
+			for_each_pipe(&data.display, pipe) {
+				for_each_valid_output_on_pipe(&data.display, pipe, output) {
+					if (!pipe_output_combo_valid(&data.display, pipe, output))
+						continue;
 
-					mode = igt_output_get_mode(output);
+					igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
+						drmModeModeInfo *mode = igt_output_get_mode(output);
 
-					test_planes_scaling_combo(&data,
+						test_planes_scaling_combo(&data,
 							get_width(mode, scaler_with_2_planes_tests[index].sf_plane1),
 							get_height(mode, scaler_with_2_planes_tests[index].sf_plane1),
 							get_width(mode, scaler_with_2_planes_tests[index].sf_plane2),
 							get_height(mode, scaler_with_2_planes_tests[index].sf_plane2),
 							pipe, output, scaler_with_2_planes_tests[index].test_type);
+					}
+					break;
+				}
+			}
+			}
+		}
+
+		for (int index = 0; index < ARRAY_SIZE(intel_paramtests); index++) {
+			igt_describe("Test for validating max source size.");
+			igt_subtest_with_dynamic(intel_paramtests[index].testname) {
+				igt_require_intel(data.drm_fd);
+				for_each_pipe(&data.display, pipe) {
+					for_each_valid_output_on_pipe(&data.display, pipe, output) {
+						drmModeModeInfo *mode = NULL;
+						/*
+						 * Need to find mode with lowest vrefresh else
+						 * we can exceed cdclk limits.
+						 */
+						mode = find_mode(&data, output, intel_paramtests[index].planesize);
+						if (mode) {
+							igt_dynamic_f("pipe-%s-%s",
+								       kmstest_pipe_name(pipe), igt_output_name(output))
+								intel_max_source_size_test(&data, pipe, output, mode,
+											   intel_paramtests[index].planesize);
+						}
+						continue;
+					}
+					break;
 				}
 			}
 		}
 
 		igt_describe("Negative test for number of scalers per pipe.");
 		igt_subtest_with_dynamic("invalid-num-scalers") {
-			for_each_pipe_with_valid_output(&data.display, pipe, output)
+			for_each_pipe_with_valid_output(&data.display, pipe, output) {
+				if (!pipe_output_combo_valid(&data.display, pipe, output))
+					continue;
+
 				igt_dynamic_f("pipe-%s-%s-invalid-num-scalers",
 					       kmstest_pipe_name(pipe), igt_output_name(output))
 					test_invalid_num_scalers(&data, pipe, output);
+			}
 		}
 	}
 
+	igt_subtest_group
+		invalid_parameter_tests(&data);
+
 	igt_describe("Tests scaling with multi-pipe.");
 	igt_subtest_f("2x-scaler-multi-pipe")
 		test_scaler_with_multi_pipe_plane(&data);
 
 	igt_fixture {
 		igt_display_fini(&data.display);
-		close(data.drm_fd);
+		drm_close_driver(data.drm_fd);
 	}
 }
