@@ -27,6 +27,11 @@
  * This is a test of drmCrtcGetSequence and drmCrtcQueueSequence
  */
 
+/**
+ * TEST: kms sequence
+ * Category: Display
+ * Description: Test CrtcGetSequence and CrtcQueueSequence.
+ */
 #include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -42,8 +47,6 @@
 
 #include <drm.h>
 
-#include "intel_bufmgr.h"
-
 IGT_TEST_DESCRIPTION("Test CrtcGetSequence and CrtcQueueSequence.");
 
 typedef struct {
@@ -58,26 +61,6 @@ typedef struct {
 #define FORKED 4
 } data_t;
 
-struct local_drm_crtc_get_sequence {
-	__u32 crtc_id;
-	__u32 active;
-	__u64 sequence;
-	__u64 sequence_ns;
-};
-
-struct local_drm_crtc_queue_sequence {
-	__u32 crtc_id;
-	__u32 flags;
-	__u64 sequence;
-	__u64 user_data;
-};
-
-#define LOCAL_DRM_IOCTL_CRTC_GET_SEQUENCE     DRM_IOWR(0x3b, struct local_drm_crtc_get_sequence)
-#define LOCAL_DRM_IOCTL_CRTC_QUEUE_SEQUENCE   DRM_IOWR(0x3c, struct local_drm_crtc_queue_sequence)
-
-#define LOCAL_DRM_CRTC_SEQUENCE_RELATIVE              0x00000001      /* sequence is relative to current */
-#define LOCAL_DRM_CRTC_SEQUENCE_NEXT_ON_MISS          0x00000002      /* Use next sequence if we've missed */
-
 struct local_drm_event_crtc_sequence {
         struct drm_event        base;
         __u64                   user_data;
@@ -99,6 +82,8 @@ static void prepare_crtc(data_t *data, int fd, igt_output_t *output)
 	igt_display_t *display = &data->display;
 	igt_plane_t *primary;
 
+	igt_display_reset(&data->display);
+
 	/* select the pipe we want to use */
 	igt_output_set_pipe(output, data->pipe);
 
@@ -106,7 +91,7 @@ static void prepare_crtc(data_t *data, int fd, igt_output_t *output)
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    LOCAL_DRM_FORMAT_MOD_NONE,
+			    DRM_FORMAT_MOD_LINEAR,
 			    0.0, 0.0, 0.0,
 			    &data->primary_fb);
 
@@ -117,7 +102,8 @@ static void prepare_crtc(data_t *data, int fd, igt_output_t *output)
 
 	igt_display_commit(display);
 
-	igt_wait_for_vblank(fd, data->pipe);
+	igt_wait_for_vblank(fd,
+			display->pipes[data->pipe].crtc_offset);
 }
 
 static void cleanup_crtc(data_t *data, int fd, igt_output_t *output)
@@ -130,88 +116,108 @@ static void cleanup_crtc(data_t *data, int fd, igt_output_t *output)
 	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 	igt_plane_set_fb(primary, NULL);
 
-	igt_output_set_pipe(output, PIPE_ANY);
-	igt_display_commit(display);
+	igt_output_set_pipe(output, PIPE_NONE);
+	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 }
 
-static int crtc_get_sequence(int fd, struct local_drm_crtc_get_sequence *cgs)
+static int crtc_get_sequence(int fd, struct drm_crtc_get_sequence *cgs)
 {
 	int err;
 
 	err = 0;
-	if (igt_ioctl(fd, LOCAL_DRM_IOCTL_CRTC_GET_SEQUENCE, cgs))
+	if (igt_ioctl(fd, DRM_IOCTL_CRTC_GET_SEQUENCE, cgs))
 		err = -errno;
 
 	return err;
 }
 
-static int crtc_queue_sequence(int fd, struct local_drm_crtc_queue_sequence *cqs)
+static int crtc_queue_sequence(int fd, struct drm_crtc_queue_sequence *cqs)
 {
 	int err;
 
 	err = 0;
-	if (igt_ioctl(fd, LOCAL_DRM_IOCTL_CRTC_QUEUE_SEQUENCE, cqs))
+	if (igt_ioctl(fd, DRM_IOCTL_CRTC_QUEUE_SEQUENCE, cqs))
 		err = -errno;
 	return err;
 }
 
+/**
+ * SUBTEST: get-%s
+ * Description: This is a test of drmCrtcGetSequence with %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @busy:         busy
+ * @forked:       forked bo
+ * @forked-busy:  forked bo & busy
+ * @idle:         idle
+ */
+
+/**
+ * SUBTEST: queue-%s
+ * Description: This is a test of drmCrtcQueueSequence with %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @busy:         busy
+ * @idle:         idle
+ */
 static void run_test(data_t *data, int fd, void (*testfunc)(data_t *, int, int))
 {
 	int nchildren =
 		data->flags & FORKED ? sysconf(_SC_NPROCESSORS_ONLN) : 1;
-	igt_display_t *display = &data->display;
-	igt_output_t *output;
-	enum pipe p;
-	unsigned int valid_tests = 0;
+	igt_output_t *output = data->output;
 
-	for_each_pipe_with_valid_output(display, p, output) {
-		data->pipe = p;
-		prepare_crtc(data, fd, output);
+	prepare_crtc(data, fd, output);
 
-		igt_info("Beginning %s on pipe %s, connector %s (%d threads)\n",
-			 igt_subtest_name(),
-			 kmstest_pipe_name(data->pipe),
-			 igt_output_name(output),
-			 nchildren);
+	igt_info("Beginning %s on pipe %s, connector %s (%d threads)\n",
+		 igt_subtest_name(),
+		 kmstest_pipe_name(data->pipe),
+		 igt_output_name(output),
+		 nchildren);
 
-		if (data->flags & BUSY) {
-			struct local_drm_crtc_queue_sequence cqs;
+	if (data->flags & BUSY) {
+		struct drm_crtc_queue_sequence cqs;
 
-			memset(&cqs, 0, sizeof(cqs));
-			cqs.crtc_id = data->crtc_id;
-			cqs.flags = LOCAL_DRM_CRTC_SEQUENCE_RELATIVE;
-			cqs.sequence = 120 + 12;
-			igt_assert_eq(crtc_queue_sequence(fd, &cqs), 0);
-		}
-
-		igt_fork(child, nchildren)
-			testfunc(data, fd, nchildren);
-		igt_waitchildren();
-
-		if (data->flags & BUSY) {
-			struct drm_event_vblank buf;
-			igt_assert_eq(read(fd, &buf, sizeof(buf)), sizeof(buf));
-		}
-
-		igt_assert(poll(&(struct pollfd){fd, POLLIN}, 1, 0) == 0);
-
-		igt_info("\n%s on pipe %s, connector %s: PASSED\n\n",
-			 igt_subtest_name(),
-			 kmstest_pipe_name(data->pipe),
-			 igt_output_name(output));
-
-		/* cleanup what prepare_crtc() has done */
-		cleanup_crtc(data, fd, output);
-		valid_tests++;
+		memset(&cqs, 0, sizeof(cqs));
+		cqs.crtc_id = data->crtc_id;
+		cqs.flags = DRM_CRTC_SEQUENCE_RELATIVE;
+		cqs.sequence = 120 + 12;
+		igt_assert_eq(crtc_queue_sequence(fd, &cqs), 0);
 	}
 
-	igt_require_f(valid_tests,
-		      "no valid crtc/connector combinations found\n");
+	igt_fork(child, nchildren)
+	testfunc(data, fd, nchildren);
+	igt_waitchildren();
+
+	if (data->flags & BUSY) {
+		struct drm_event_vblank buf;
+
+		igt_assert_eq(read(fd, &buf, sizeof(buf)), sizeof(buf));
+	}
+
+	igt_assert(poll(&(struct pollfd){fd, POLLIN}, 1, 0) == 0);
+
+	igt_info("\n%s on pipe %s, connector %s: PASSED\n\n",
+		 igt_subtest_name(),
+		 kmstest_pipe_name(data->pipe),
+		 igt_output_name(output));
+
+	/* cleanup what prepare_crtc() has done */
+	cleanup_crtc(data, fd, output);
 }
 
 static void sequence_get(data_t *data, int fd, int nchildren)
 {
-	struct local_drm_crtc_get_sequence cgs;
+	struct drm_crtc_get_sequence cgs;
 	struct timespec start, end;
 	unsigned long sq, count = 0;
 
@@ -234,8 +240,8 @@ static void sequence_get(data_t *data, int fd, int nchildren)
 
 static void sequence_queue(data_t *data, int fd, int nchildren)
 {
-	struct local_drm_crtc_get_sequence cgs_start, cgs_end;
-	struct local_drm_crtc_queue_sequence cqs;
+	struct drm_crtc_get_sequence cgs_start, cgs_end;
+	struct drm_crtc_queue_sequence cqs;
 	unsigned long target;
 	int total = 120 / nchildren;
 	int n;
@@ -274,6 +280,8 @@ static void sequence_queue(data_t *data, int fd, int nchildren)
 igt_main
 {
 	int fd;
+	igt_output_t *output;
+	enum pipe p;
 	data_t data;
 	const struct {
 		const char *name;
@@ -299,6 +307,7 @@ igt_main
 		fd = drm_open_driver_master(DRIVER_ANY);
 		kmstest_set_vt_graphics_mode();
 		igt_display_require(&data.display, fd);
+		igt_display_require_output(&data.display);
 	}
 
 	for (f = funcs; f->name; f++) {
@@ -306,10 +315,28 @@ igt_main
 			if (m->flags & ~f->valid)
 				continue;
 
-			igt_subtest_f("%s-%s", f->name, m->name) {
-				data.flags = m->flags;
-				run_test(&data, fd, f->func);
+			igt_describe("This is a test of drmCrtcGetSequence and "
+				     "drmCrtcQueueSequence");
+			igt_subtest_with_dynamic_f("%s-%s", f->name, m->name) {
+				for_each_pipe_with_valid_output(&data.display, p, output) {
+					igt_display_reset(&data.display);
+
+					igt_output_set_pipe(output, p);
+					if (!i915_pipe_output_combo_valid(&data.display))
+						continue;
+
+					igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(p)) {
+						data.pipe = p;
+						data.output = output;
+						data.flags = m->flags;
+						run_test(&data, fd, f->func);
+					}
+				}
 			}
 		}
 	}
+
+	igt_fixture {
+		drm_close_driver(fd);
+	}
 }
