@@ -40,6 +40,157 @@
 #include <sys/wait.h>
 
 #include <drm.h>
+/**
+ * TEST: kms vblank
+ * Category: Display
+ * Description: Test speed of WaitVblank.
+ *
+ * SUBTEST: crtc-id
+ * Description: check the Vblank and flip events works with given crtc id
+ * Driver requirement: i915, xe
+ * Functionality: vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: invalid
+ * Description: Negative test for vblank request
+ * Driver requirement: i915, xe
+ * Functionality: vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
+/**
+ * SUBTEST: pipe-%s-ts-continuation-dpms-rpm
+ * Description: Test TS continuty with DPMS & RPM while hanging by introducing
+ *              NOHANG flag on %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: dpms, hang, rpm, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @A:     pipe A
+ * @B:     pipe B
+ * @C:     pipe C
+ * @D:     pipe D
+ * @E:     pipe E
+ * @F:     pipe F
+ * @G:     pipe G
+ * @H:     pipe H
+ */
+
+/**
+ * SUBTEST: pipe-%s-ts-continuation-dpms-suspend
+ * Description: Test TS continuty with DPMS & Suspend while hanging by introducing
+ *              NOHANG flag on %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: dpms, hang, suspend, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @A:     pipe A
+ * @B:     pipe B
+ * @C:     pipe C
+ * @D:     pipe D
+ * @E:     pipe E
+ * @F:     pipe F
+ * @G:     pipe G
+ * @H:     pipe H
+ */
+
+/**
+ * SUBTEST: pipe-%s-ts-continuation-suspend
+ * Description: Test TS continuty with Suspend while hanging by introducing NOHANG
+ *              flag on %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: hang, suspend, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @A:     pipe A
+ * @B:     pipe B
+ * @C:     pipe C
+ * @D:     pipe D
+ * @E:     pipe E
+ * @F:     pipe F
+ * @G:     pipe G
+ * @H:     pipe H
+ */
+
+/**
+ * SUBTEST: pipe-%s-ts-continuation-modeset-rpm
+ * Description: Test TS continuty during Modeset with Suspend while hanging by
+ *              introducing NOHANG flag on %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: hang, rpm, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @A:     pipe A
+ * @B:     pipe B
+ * @C:     pipe C
+ * @D:     pipe D
+ * @E:     pipe E
+ * @F:     pipe F
+ * @G:     pipe G
+ * @H:     pipe H
+ */
+
+/**
+ * SUBTEST: pipe-%s-accuracy-idle
+ * Description: Test Accuracy of vblank events while hanging by introducing NOHANG
+ *              flag on %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: hang, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: pipe-%s-%s
+ * Description: Test %arg[2] while hanging by introducing NOHANG flag on %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: hang, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * SUBTEST: pipe-%s-%s-hang
+ * Description: Test %arg[2] with injected hang is working properly on %arg[1]
+ * Driver requirement: i915, xe
+ * Functionality: hang, vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ *
+ * arg[1]:
+ *
+ * @A:     pipe A
+ * @B:     pipe B
+ * @C:     pipe C
+ * @D:     pipe D
+ * @E:     pipe E
+ * @F:     pipe F
+ * @G:     pipe G
+ * @H:     pipe H
+ *
+ * arg[2]:
+ *
+ * @query-idle:              Time taken to Query vblank counters
+ * @query-forked:            Time taken to Query vblank counters (multithreaded)
+ * @query-busy:              Time taken to Query vblank counters (during V-active)
+ * @query-forked-busy:       Time taken to Query vblank counters (during V-active mutithreaded)
+ * @wait-idle:               Time taken to wait for vblanks
+ * @wait-forked:             Time taken to wait for vblanks (multithreaded)
+ * @wait-busy:               Time taken to wait for vblanks (during V-active)
+ * @wait-forked-busy:        Time taken to wait for vblanks (during V-active mutithreaded)
+ * @ts-continuation-idle:    TS continuty
+ * @ts-continuation-modeset: TS continuty during modeset
+ */
 
 IGT_TEST_DESCRIPTION("Test speed of WaitVblank.");
 
@@ -79,11 +230,10 @@ static void prepare_crtc(data_t *data, int fd, igt_output_t *output)
 
 	/* create and set the primary plane fb */
 	mode = igt_output_get_mode(output);
-	igt_create_color_fb(fd, mode->hdisplay, mode->vdisplay,
-			    DRM_FORMAT_XRGB8888,
-			    DRM_FORMAT_MOD_LINEAR,
-			    0.0, 0.0, 0.0,
-			    &data->primary_fb);
+	igt_create_fb(fd, mode->hdisplay, mode->vdisplay,
+		      DRM_FORMAT_XRGB8888,
+		      DRM_FORMAT_MOD_LINEAR,
+		      &data->primary_fb);
 
 	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 	igt_plane_set_fb(primary, &data->primary_fb);
@@ -130,7 +280,10 @@ static void run_test(data_t *data, void (*testfunc)(data_t *, int, int))
 		 igt_output_name(output));
 
 	if (!(data->flags & NOHANG)) {
-		ahnd = get_reloc_ahnd(fd, 0);
+		ahnd = is_i915_device(fd) ?
+			get_reloc_ahnd(fd, 0) :
+			intel_allocator_open(fd, 0, INTEL_ALLOCATOR_RELOC);
+
 		hang = igt_hang_ring_with_ahnd(fd, I915_EXEC_DEFAULT, ahnd);
 	}
 
@@ -175,6 +328,22 @@ static void run_test(data_t *data, void (*testfunc)(data_t *, int, int))
 	cleanup_crtc(data, fd, output);
 }
 
+static bool
+pipe_output_combo_valid(igt_display_t *display,
+			enum pipe pipe, igt_output_t *output)
+{
+	bool ret = true;
+
+	igt_display_reset(display);
+
+	igt_output_set_pipe(output, pipe);
+	if (!i915_pipe_output_combo_valid(display))
+		ret = false;
+	igt_output_set_pipe(output, PIPE_NONE);
+
+	return ret;
+}
+
 static void crtc_id_subtest(data_t *data, int fd)
 {
 	igt_display_t *display = &data->display;
@@ -188,6 +357,15 @@ static void crtc_id_subtest(data_t *data, int fd)
 		uint64_t val;
 		union drm_wait_vblank vbl;
 
+		igt_display_reset(display);
+
+		igt_output_set_pipe(output, p);
+		if (!i915_pipe_output_combo_valid(display))
+			continue;
+
+		igt_info("Using (pipe %s + %s) to run the subtest.\n",
+			 kmstest_pipe_name(p), igt_output_name(output));
+
 		crtc_id = display->pipes[p].crtc_id;
 		if (drmGetCap(display->drm_fd, DRM_CAP_CRTC_IN_VBLANK_EVENT, &val) == 0)
 			expected_crtc_id = crtc_id;
@@ -449,10 +627,18 @@ static void run_subtests_for_pipe(data_t *data)
 			igt_subtest_f("pipe-%s-%s-%s",
 				      kmstest_pipe_name(data->pipe),
 				      f->name, m->name) {
+				int found = 0;
+
 				for_each_valid_output_on_pipe(&data->display, data->pipe, data->output) {
+					if (!pipe_output_combo_valid(&data->display, data->pipe, data->output))
+						continue;
+
 					data->flags = m->flags | NOHANG;
 					run_test(data, f->func);
+
+					found++;
 				}
+				igt_require_f(found, "No valid pipe/output combo found.\n");
 			}
 
 			/* Skip the -hang version if NOHANG flag is set */
@@ -464,13 +650,20 @@ static void run_subtests_for_pipe(data_t *data)
 				      kmstest_pipe_name(data->pipe),
 				      f->name, m->name) {
 				igt_hang_t hang;
+				int found = 0;
 
 				hang = igt_allow_hang(data->display.drm_fd, 0, 0);
 				for_each_valid_output_on_pipe(&data->display, data->pipe, data->output) {
+					if (!pipe_output_combo_valid(&data->display, data->pipe, data->output))
+						continue;
+
 					data->flags = m->flags;
 					run_test(data, f->func);
+
+					found++;
 				}
 				igt_disallow_hang(data->display.drm_fd, hang);
+				igt_require_f(found, "No valid pipe/output combo found.\n");
 			}
 		}
 	}
@@ -482,12 +675,22 @@ static void invalid_subtest(data_t *data, int fd)
 	unsigned long valid_flags;
 	igt_display_t* display = &data->display;
 	enum pipe pipe = 0;
-	igt_output_t* output = igt_get_single_output_for_pipe(display, pipe);
+	igt_output_t *output;
+
+	igt_display_reset(display);
+
+	output = igt_get_single_output_for_pipe(display, pipe);
+	igt_require(output);
 
 	data->pipe = pipe;
 	data->output = output;
+
 	igt_output_set_pipe(output, pipe);
-	igt_display_require_output_on_pipe(display, pipe);
+	igt_require(i915_pipe_output_combo_valid(display));
+
+	igt_info("Using (pipe %s + %s) to run the subtest.\n",
+		 kmstest_pipe_name(pipe), igt_output_name(output));
+
 	prepare_crtc(data, fd, output);
 
 	/* First check all is well with a simple query */
@@ -552,6 +755,6 @@ igt_main
 			run_subtests_for_pipe(&data);
 
 	igt_fixture {
-		close(fd);
+		drm_close_driver(fd);
 	}
 }
