@@ -30,6 +30,132 @@
 #include "igt.h"
 #include "igt_vgem.h"
 #include "intel_batchbuffer.h"	/* igt_blitter_src_copy() */
+/**
+ * TEST: prime vgem
+ * Description: Basic check of polling for prime/vgem fences.
+ *
+ * SUBTEST: basic-blt
+ * Category: Infrastructure
+ * Description: Examine blitter access path.
+ * Feature: prime
+ * Functionality: mock device
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: basic-fence-blt
+ * Category: Infrastructure
+ * Description: Examine blitter access path fencing.
+ * Feature: prime
+ * Functionality: mock device
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: basic-fence-flip
+ * Description: Examine vgem bo front/back flip fencing.
+ * Feature: prime
+ * Run type: BAT
+ *
+ * SUBTEST: basic-fence-mmap
+ * Description: Examine GTT access path fencing.
+ * Feature: gtt, prime
+ * Run type: BAT
+ *
+ * SUBTEST: basic-fence-read
+ * Description: Examine read access path fencing.
+ * Feature: gtt, prime
+ * Run type: BAT
+ *
+ * SUBTEST: basic-gtt
+ * Description: Examine access path through GTT.
+ * Feature: gtt, prime
+ * Run type: BAT
+ *
+ * SUBTEST: basic-read
+ * Description: Examine read access path.
+ * Feature: gtt, prime
+ * Run type: BAT
+ *
+ * SUBTEST: basic-write
+ * Description: Examine write access path.
+ * Feature: gtt, prime
+ * Run type: BAT
+ *
+ * SUBTEST: busy
+ * Category: Infrastructure
+ * Description: Examine busy check of polling for vgem fence.
+ * Feature: prime
+ * Functionality: mock device
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: coherency-blt
+ * Category: Infrastructure
+ * Description: Examine blitter access path WC coherency.
+ * Feature: prime
+ * Functionality: mock device
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: coherency-gtt
+ * Description: Examine concurrent access of vgem bo.
+ * Feature: gtt, prime
+ * Run type: FULL
+ *
+ * SUBTEST: fence-flip-hang
+ * Description: Examine vgem bo front/back flip fencing with a pending gpu hang.
+ * Feature: blacklist, prime, synchronization
+ * Run type: FULL
+ *
+ * SUBTEST: fence-read-hang
+ * Description: Examine read access path fencing with a pending gpu hang.
+ * Feature: blacklist, prime, synchronization
+ * Run type: FULL
+ *
+ * SUBTEST: fence-wait
+ * Category: Infrastructure
+ * Description: Examine basic dma-buf fence interop.
+ * Feature: prime, synchronization
+ * Functionality: mock device
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: fence-write-hang
+ * Description: Examine write access path fencing with a pending gpu hang.
+ * Feature: blacklist, prime, synchronization
+ * Run type: FULL
+ *
+ * SUBTEST: shrink
+ * Category: Infrastructure
+ * Description: Examine link establishment between shrinker and vgem bo.
+ * Feature: prime
+ * Functionality: mock device
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: sync
+ * Category: Infrastructure
+ * Description: Examine sync on vgem fence.
+ * Feature: prime
+ * Functionality: mock device
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: wait
+ * Category: Infrastructure
+ * Description: Examine wait on vgem fence.
+ * Feature: prime
+ * Functionality: mock device
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ */
 
 IGT_TEST_DESCRIPTION("Basic check of polling for prime/vgem fences.");
 
@@ -294,50 +420,6 @@ static void test_write(int vgem, int i915)
 	munmap(ptr, scratch.size);
 }
 
-static bool has_userptr(int i915)
-{
-	uint32_t handle = 0;
-	void *ptr;
-
-	igt_assert(posix_memalign(&ptr, 4096, 4096) == 0);
-	if (__gem_userptr(i915, ptr, 4096, 0, 0, &handle) == 0)
-		gem_close(i915, handle);
-	free(ptr);
-
-	return handle;
-}
-
-static void test_userptr(int vgem, int i915)
-{
-	struct drm_i915_gem_exec_object2 obj = {};
-	struct drm_i915_gem_execbuffer2 execbuf = {
-		.buffers_ptr = to_user_pointer(&obj),
-		.buffer_count = 1,
-	};
-	struct vgem_bo scratch;
-	uint32_t *ptr;
-
-	igt_require(has_userptr(i915));
-
-	scratch.width = 1024;
-	scratch.height = 1024;
-	scratch.bpp = 32;
-	vgem_create(vgem, &scratch);
-
-	/* vgem is acting as a simple provider of a single vma backed by pages */
-	ptr = vgem_mmap(vgem, &scratch, PROT_WRITE);
-	gem_close(vgem, scratch.handle);
-	*ptr = MI_BATCH_BUFFER_END;
-
-	gem_userptr(i915, ptr, scratch.size, 0, 0, &obj.handle);
-	gem_set_caching(i915, obj.handle, I915_CACHING_NONE); /* for !llc exec */
-
-	gem_execbuf(i915, &execbuf);
-	gem_close(i915, obj.handle);
-
-	munmap(ptr, scratch.size);
-}
-
 static void test_gtt(int vgem, int i915)
 {
 	struct vgem_bo scratch;
@@ -624,7 +706,7 @@ static void work(int i915, uint64_t ahnd, uint64_t scratch_offset, int dmabuf,
 		store[count].delta = sizeof(uint32_t) * count;
 		store[count].read_domains = I915_GEM_DOMAIN_INSTRUCTION;
 		store[count].write_domain = I915_GEM_DOMAIN_INSTRUCTION;
-		batch[i] = MI_STORE_DWORD_IMM | (gen < 6 ? 1 << 22 : 0);
+		batch[i] = MI_STORE_DWORD_IMM_GEN4 | (gen < 6 ? 1 << 22 : 0);
 		if (gen >= 8) {
 			batch[++i] = scratch_offset + store[count].delta;
 			batch[++i] = (scratch_offset + store[count].delta) >> 32;
@@ -1093,7 +1175,10 @@ static void test_each_engine(const char *name, int vgem, int i915,
 					unsigned int flags))
 {
 	const struct intel_execution_engine2 *e;
-	const intel_ctx_t *ctx = intel_ctx_create_all_physical(i915);
+	const intel_ctx_t *ctx;
+
+	igt_fixture
+		ctx = intel_ctx_create_all_physical(i915);
 
 	igt_subtest_with_dynamic(name) {
 		for_each_ctx_engine(i915, ctx, e) {
@@ -1110,7 +1195,8 @@ static void test_each_engine(const char *name, int vgem, int i915,
 		}
 	}
 
-	intel_ctx_destroy(i915, ctx);
+	igt_fixture
+		intel_ctx_destroy(i915, ctx);
 }
 
 igt_main
@@ -1136,10 +1222,6 @@ igt_main
 	igt_subtest("basic-write")
 		test_write(vgem, i915);
 
-	igt_describe("Check that we wrap the vgem mmap with userptr.");
-	igt_subtest("basic-userptr")
-		test_userptr(vgem, i915);
-
 	igt_describe("Examine access path through GTT.");
 	igt_subtest("basic-gtt") {
 		gem_require_mappable_ggtt(i915);
@@ -1244,7 +1326,7 @@ igt_main
 
 
 	igt_fixture {
-		close(i915);
-		close(vgem);
+		drm_close_driver(i915);
+		drm_close_driver(vgem);
 	}
 }
