@@ -38,6 +38,52 @@
 
 #include "sw_sync.h"
 
+/**
+ * TEST: sw sync
+ * Category: Infrastructure
+ * Description: Test SW Sync Framework
+ * Feature: synchronization
+ * Functionality: semaphore
+ * Run type: FULL
+ * Sub-category: DRM
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: alloc_fence
+ *
+ * SUBTEST: alloc_fence_invalid_timeline
+ *
+ * SUBTEST: alloc_merge_fence
+ *
+ * SUBTEST: alloc_timeline
+ *
+ * SUBTEST: sync_busy
+ *
+ * SUBTEST: sync_busy_fork
+ *
+ * SUBTEST: sync_busy_fork_unixsocket
+ *
+ * SUBTEST: sync_expired_merge
+ *
+ * SUBTEST: sync_merge
+ *
+ * SUBTEST: sync_merge_invalid
+ *
+ * SUBTEST: sync_merge_same
+ *
+ * SUBTEST: sync_multi_consumer
+ *
+ * SUBTEST: sync_multi_consumer_producer
+ *
+ * SUBTEST: sync_multi_producer_single_consumer
+ *
+ * SUBTEST: sync_multi_timeline_wait
+ *
+ * SUBTEST: sync_random_merge
+ *
+ * SUBTEST: timeline_closed
+ *
+ * SUBTEST: timeline_closed_signaled
+ */
 
 IGT_TEST_DESCRIPTION("Test SW Sync Framework");
 
@@ -837,8 +883,9 @@ igt_main
 	igt_fixture {
 		igt_require_sw_sync();
 		multi_consumer_threads =
-			min(multi_consumer_threads,
-			    sysconf(_SC_NPROCESSORS_ONLN));
+			min_t(multi_consumer_threads,
+			      multi_consumer_threads,
+			      sysconf(_SC_NPROCESSORS_ONLN));
 	}
 
 	igt_subtest("alloc_timeline")
