@@ -69,6 +69,19 @@
 #include <stdlib.h>
 #include <signal.h>
 
+/**
+ * TEST: testdisplay
+ * Category: Display
+ * Description: Tests basic display functionality.
+ *
+ * SUBTEST:
+ * Description: This test is intended for testing of display functionality like
+ *              modeset, clone modes, test patterns & pixel generators etc..
+ * Driver requirement: i915, xe
+ * Functionality: kms_core
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 IGT_TEST_DESCRIPTION("Tests basic display functionality.");
 
 enum {
@@ -548,7 +561,7 @@ int update_display(bool probe)
 
 __noreturn static void cleanup_and_exit(int ret)
 {
-	close(drm_fd);
+	drm_close_driver(drm_fd);
 	exit(ret);
 }
 
@@ -779,7 +792,7 @@ out_hotplug:
 out_mainloop:
 	g_main_loop_unref(mainloop);
 out_close:
-	close(drm_fd);
+	drm_close_driver(drm_fd);
 
 	igt_assert_eq(ret, 0);
 }
